----------------------------------------------------------------------------

----------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY adder128 IS 
  PORT( 
  x, y: IN STD_LOGIC_VECTOR(127 DOWNTO 0);
  c_in: IN STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(127 DOWNTO 0);
  c_out: OUT STD_LOGIC
  );
END adder128;

ARCHITECTURE circuit OF adder128 IS
   
  SIGNAL carries: STD_LOGIC_VECTOR(4 DOWNTO 0);
    
  COMPONENT carry_select_adder IS
    GENERIC(k, m: NATURAL);
  PORT(
    x, y: IN STD_LOGIC_VECTOR(k*m-1 DOWNTO 0);
    c_in: IN STD_LOGIC;
    z: OUT STD_LOGIC_VECTOR(k*m-1 DOWNTO 0);
    c_out: OUT STD_LOGIC
  );
  END COMPONENT;
  
BEGIN

--  first_adder: carry_select_adder GENERIC MAP(k => 8, m => 4)
--  PORT MAP(x => x, y => y, c_in => c_in, z => z, c_out => c_out);

  carries(0) <= c_in;
  iteration: FOR i IN 0 TO 3 GENERATE
    main_component: carry_select_adder GENERIC MAP(k => 8, m => 4)
    PORT MAP(x => x(32*i+31 DOWNTO 32*i), y => y(32*i+31 DOWNTO 32*i), c_in => carries(i),
       z => z(32*i+31 DOWNTO 32*i), c_out => carries(i+1));
  END GENERATE;
  c_out <= carries(4);

END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY test_adder128 IS END test_adder128;

ARCHITECTURE test OF test_adder128 IS

COMPONENT adder128 IS 
  PORT( 
  x, y: IN STD_LOGIC_VECTOR(127 DOWNTO 0);
  c_in: IN STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(127 DOWNTO 0);
  c_out: OUT STD_LOGIC
  );
END COMPONENT;

SIGNAL x, y, z: STD_LOGIC_VECTOR(127 DOWNTO 0);
SIGNAL c_in, c_out: STD_LOGIC;

BEGIN

dut: adder128 PORT MAP ( x => x, y => y, z => z, c_in => c_in, c_out => c_out);

x <= x"0123456789abcdef0123456789abcdef";
y <= x"fedcba9876543210fedcba9876543210";
c_in <= '0', '1' AFTER 100 NS;

END test;