----------------------------------------------------------------------------

----------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY adder128pipeline IS 
  PORT( 
  x, y: IN STD_LOGIC_VECTOR(127 DOWNTO 0);
  clk, c_in: IN STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(127 DOWNTO 0);
  c_out: OUT STD_LOGIC
  );
END adder128pipeline;

ARCHITECTURE circuit OF adder128pipeline IS
   
  SIGNAL c_in1, c_in2, c_in3, c_out0, c_out1, c_out2, c_out3: STD_LOGIC;
  SIGNAL x1, y1: STD_LOGIC_VECTOR(127 DOWNTO 32); 
  SIGNAL x2, y2: STD_LOGIC_VECTOR(127 DOWNTO 64); 
  SIGNAL x3, y3: STD_LOGIC_VECTOR(127 DOWNTO 96);
  SIGNAL z0: STD_LOGIC_VECTOR(31 DOWNTO 0);  
  SIGNAL z1: STD_LOGIC_VECTOR(63 DOWNTO 0);  
  SIGNAL z2: STD_LOGIC_VECTOR(95 DOWNTO 0);  
  SIGNAL z3: STD_LOGIC_VECTOR(127 DOWNTO 0);  
    
  COMPONENT carry_select_adder IS
    GENERIC(k, m: NATURAL);
  PORT(
    x, y: IN STD_LOGIC_VECTOR(k*m-1 DOWNTO 0);
    c_in: IN STD_LOGIC;
    z: OUT STD_LOGIC_VECTOR(k*m-1 DOWNTO 0);
    c_out: OUT STD_LOGIC
  );
  END COMPONENT;
  
BEGIN

  stage0: carry_select_adder GENERIC MAP(k => 8, m => 4)
  PORT MAP(x => x(31 DOWNTO 0), y => y(31 DOWNTO 0), c_in => c_in, 
    z => z0, c_out => c_out0);

  stage1: carry_select_adder GENERIC MAP(k => 8, m => 4)
  PORT MAP(x => x1(63 DOWNTO 32), y => y1(63 DOWNTO 32), c_in => c_in1, 
    z => z1(63 DOWNTO 32), c_out => c_out1);

  stage2: carry_select_adder GENERIC MAP(k => 8, m => 4)
  PORT MAP(x => x2(95 DOWNTO 64), y => y2(95 DOWNTO 64), c_in => c_in2, 
    z => z2(95 DOWNTO 64), c_out => c_out2);

  stage3: carry_select_adder GENERIC MAP(k => 8, m => 4)
  PORT MAP(x => x3(127 DOWNTO 96), y => y3(127 DOWNTO 96), c_in => c_in3, 
    z => z3(127 DOWNTO 96), c_out => c_out3);

  registers: PROCESS(clk)
  BEGIN
    IF clk'EVENT AND clk = '1' THEN
      x1 <= x(127 DOWNTO 32); y1 <= y(127 DOWNTO 32); c_in1 <= c_out0; z1(31 DOWNTO 0) <= z0; 
      x2 <= x1(127 DOWNTO 64); y2 <= y1(127 DOWNTO 64); c_in2 <= c_out1; z2(63 DOWNTO 0) <= z1; 
      x3 <= x2(127 DOWNTO 96); y3 <= y2(127 DOWNTO 96); c_in3 <= c_out2; z3(95 DOWNTO 0) <= z2; 
      z <= z3; c_out <= c_out3;
   END IF;
  END PROCESS registers;

--  carries(0) <= c_in;
--  iteration: FOR i IN 0 TO 3 GENERATE
--    main_component: carry_select_adder GENERIC MAP(k => 8, m => 4)
--    PORT MAP(x => x(32*i+31 DOWNTO 32*i), y => y(32*i+31 DOWNTO 32*i), c_in => carries(i),
--       z => z(32*i+31 DOWNTO 32*i), c_out => carries(i+1));
--  END GENERATE;
--  c_out <= carries(4);
--
END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY test_adder128pipeline IS END test_adder128pipeline;

ARCHITECTURE test OF test_adder128pipeline IS

COMPONENT adder128pipeline IS 
  PORT( 
  x, y: IN STD_LOGIC_VECTOR(127 DOWNTO 0);
  clk, c_in: IN STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(127 DOWNTO 0);
  c_out: OUT STD_LOGIC
  );
END COMPONENT;

SIGNAL x, y, z: STD_LOGIC_VECTOR(127 DOWNTO 0);
SIGNAL c_in, c_out: STD_LOGIC;
SIGNAL clk: STD_LOGIC := '0';

BEGIN

dut: adder128pipeline PORT MAP ( x => x, y => y, z => z, c_in => c_in, c_out => c_out, clk => clk);

clk <= NOT(clk) AFTER 50 NS;

PROCESS
BEGIN
x <= x"0123456789abcdef0123456789abcdee";
y <= x"fedcba9876543210fedcba9876543210";
c_in <= '1';
WAIT FOR 50 ns;
x <= x"11111111111111111111111111111110";
y <= x"00000000000000000000000000000000";
WAIT FOR 100 ns;
x <= x"88888888888888888888888888888888";
y <= x"99999999999999999999999999999999";
WAIT FOR 100 ns;
x <= x"bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb";
y <= x"44444444444444444444444444444444";
WAIT FOR 100 ns;
x <= x"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee";
y <= x"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
WAIT FOR 100 ns;
x <= x"ffffffffffffffffffffffffffffffff";
y <= x"ffffffffffffffffffffffffffffffff";
WAIT;
END PROCESS;



--x <= x"0123456789abcdef0123456789abcdee", 
--     x"11111111111111111111111111111110" AFTER 149 NS,  
--     x"88888888888888888888888888888888" AFTER 249 NS,  
--     x"bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb" AFTER 349 NS,  
--     x"eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" AFTER 449 NS,  
--     x"ffffffffffffffffffffffffffffffff" AFTER 549 NS;  
--
--y <= x"fedcba9876543210fedcba9876543210",
--     x"00000000000000000000000000000000" AFTER 149 NS,  
--     x"99999999999999999999999999999999" AFTER 249 NS,  
--     x"44444444444444444444444444444444" AFTER 349 NS, 
--     x"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" AFTER 449 NS,  
--     x"ffffffffffffffffffffffffffffffff" AFTER 549 NS;
--c_in <= '1';
--
END test;