LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY alu_kernel IS
  GENERIC(m: NATURAL:= 32);
PORT (
  x, y: IN STD_LOGIC_VECTOR(m-1 DOWNTO 0);
  oper: IN STD_LOGIC;
  sel: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  z: INOUT STD_LOGIC_VECTOR(m-1 DOWNTO 0);
  overflow, negative: OUT STD_LOGIC;
  carry, zero: INOUT STD_LOGIC
);
END alu_kernel;

ARCHITECTURE structure OF alu_kernel IS

SIGNAL c, s, g, p: STD_LOGIC_VECTOR(m-1 DOWNTO 0);
CONSTANT vector_zero: STD_LOGIC_VECTOR(m-1 DOWNTO 0) := (OTHERS => '0');

BEGIN
 
c(0) <= oper;
iteration: FOR i IN 0 TO m-2 GENERATE
   g(i) <= x(i) AND (y(i) XOR oper);
   p(i) <= x(i) OR (y(i) XOR oper);
   s(i) <= x(i) XOR (y(i) XOR oper) XOR c(i);
   c(i+1) <= g(i) OR (p(i) AND c(i));
   WITH sel SELECT z(i) <= s(i) WHEN "00", g(i) WHEN "01", p(i) WHEN OTHERS;
END GENERATE;

g(m-1) <= x(m-1) AND (y(m-1) XOR oper);
p(m-1) <= x(m-1) OR (y(m-1) XOR oper);
s(m-1) <= x(m-1) XOR (y(m-1) XOR oper) XOR c(m-1);
carry <= g(m-1) OR (p(m-1) AND c(m-1));
overflow <= carry XOR c(m-1);
WITH sel SELECT z(m-1) <= s(m-1) WHEN "00", g(m-1) WHEN "01", p(m-1) WHEN OTHERS;

negative <= z(m-1);

nor_function: PROCESS(z)
BEGIN
   IF z = vector_zero THEN zero <= '1'; ELSE zero <= '0'; END IF;
END PROCESS;
END structure;






LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_alu_kernel IS END test_alu_kernel;

ARCHITECTURE test OF test_alu_kernel IS

COMPONENT alu_kernel IS
  GENERIC(m: NATURAL:= 32);
PORT (
  x, y: IN STD_LOGIC_VECTOR(m-1 DOWNTO 0);
  oper: IN STD_LOGIC;
  sel: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  z: INOUT STD_LOGIC_VECTOR(m-1 DOWNTO 0);
  overflow, negative: OUT STD_LOGIC;
  carry, zero: INOUT STD_LOGIC
);
END COMPONENT;

SIGNAL x, y, z: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL oper, overflow, negative, carry, zero: STD_LOGIC;
SIGNAL sel: STD_LOGIC_VECTOR(1 DOWNTO 0);

BEGIN

dut: alu_kernel GENERIC MAP (m => 8)
   PORT MAP (x => x, y => y, oper => oper, sel => sel, z => z,
     overflow => overflow, negative => negative, carry => carry, zero => zero);

test_vectors: PROCESS
BEGIN
oper <= '0';
sel <= "00";
x <= x"a9";
y <= x"b7";
WAIT FOR 100 NS;
oper <= '1';
WAIT FOR 100 NS;
sel <= "01";
WAIT FOR 100 NS;
sel <= "10";
WAIT FOR 100 NS;
oper <= '0';
WAIT FOR 100 NS;
sel <= "01";
WAIT FOR 100 NS;
sel <= "10";
WAIT FOR 100 NS;
oper <= '1';
sel <= "00";
x <= x"80";
y <= x"7f";
WAIT FOR 100 NS;
y <= x"80";
WAIT;
END PROCESS;
END test;