----------------------------------------------------------------------------


----------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY bus_interface IS
PORT (
    address_bus: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    clk, reset, write_slave, ADS: IN STD_LOGIC;
    address: OUT STD_LOGIC_VECTOR(13 DOWNTO 0);
    WE, OE: OUT STD_LOGIC
);
END bus_interface;

ARCHITECTURE data_flow OF bus_interface IS

   TYPE state IS RANGE 0 TO 2;
   SIGNAL current_state: state;


BEGIN

address <= address_bus(13 DOWNTO 0);

next_state: PROCESS(reset, clk)
BEGIN
  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'event and clk = '1' THEN
    CASE current_state IS
        WHEN 0 => IF (ADS AND address_bus(15) AND NOT(address_bus(14)) AND write_slave) = '1' 
                  THEN current_state <= 1;
                  ELSIF (ADS AND address_bus(15) AND NOT(address_bus(14)) AND NOT(write_slave)) = '1'
                  THEN current_state <= 2; END IF;
        WHEN 1 TO 2 => current_state <= 0;
      END CASE;
    END IF;
  END PROCESS next_state;

output_state: PROCESS(current_state)
 BEGIN
   CASE current_state IS
     WHEN 0 => WE <= '0'; OE <= '0';
     WHEN 1 => WE <= '1'; OE <= '0';
     WHEN 2 => WE <= '0'; OE <= '1';
   END CASE;
 END PROCESS output_state;


END data_flow;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_bus_interface IS END test_bus_interface;

ARCHITECTURE test OF test_bus_interface IS

COMPONENT bus_interface IS
PORT (
    address_bus: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    clk, reset, write_slave, ADS: IN STD_LOGIC;
    address: OUT STD_LOGIC_VECTOR(13 DOWNTO 0);
    WE, OE: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL address_bus: STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL clk: STD_LOGIC := '1';
SIGNAL reset, write_slave, ADS, WE, OE: STD_LOGIC;
SIGNAL address: STD_LOGIC_VECTOR(13 DOWNTO 0);

BEGIN

dut: bus_interface 
PORT MAP (address_bus => address_bus, clk => clk,  reset => reset,  write_slave => write_slave,
ADS => ADS, address => address, WE => WE, OE => OE);

clk <= NOT(clk) AFTER 50 NS;

PROCESS
BEGIN
address_bus <= x"0000"; write_slave <= '0'; ADS <= '0';
reset <= '1'; 
WAIT FOR 50 NS; 
reset <= '0'; 
WAIT FOR 350 NS;
-------------------
address_bus <= x"9c2b"; write_slave <= '1'; ADS <= '1';
WAIT FOR 100 NS;
ADS <= '0';
WAIT FOR 400 NS;
--------------------
address_bus <= x"77d4"; write_slave <= '0'; ADS <= '1';
WAIT FOR 100 NS;
ADS <= '0';
WAIT FOR 400 NS;
--------------------
address_bus <= x"a7d4"; write_slave <= '0'; ADS <= '1';
WAIT FOR 100 NS;
ADS <= '0';
WAIT FOR 400 NS;
--------------------
WAIT;
END PROCESS;
END test;
 




