----------------------------------------------------------------------------


----------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY bus_interface_2 IS
PORT (
    address: IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    clk, reset, write_slave, ADS, done: IN STD_LOGIC;
    slave_address: OUT STD_LOGIC_VECTOR(29 DOWNTO 0);
    WE, ME, ready: OUT STD_LOGIC
);
END bus_interface_2;

ARCHITECTURE data_flow OF bus_interface_2 IS

   TYPE state IS RANGE 0 TO 4;
   SIGNAL current_state: state;


BEGIN

slave_address <= address(29 DOWNTO 0);

next_state: PROCESS(reset, clk)
BEGIN
  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'event and clk = '1' THEN
    CASE current_state IS
        WHEN 0 => IF (ADS AND address(31) AND NOT(address(30)) AND write_slave) = '1' 
                  THEN current_state <= 1;
                  ELSIF (ADS AND address(31) AND NOT(address(30)) AND NOT(write_slave)) = '1'
                  THEN current_state <= 2; END IF;
        WHEN 1 TO 2 => current_state <= 3;
        WHEN 3 => IF done = '1' THEN current_state <= 4; END IF;
        WHEN 4 => current_state <= 0;
      END CASE;
    END IF;
  END PROCESS next_state;

output_state: PROCESS(current_state)
 BEGIN
   CASE current_state IS
     WHEN 0 => WE <= '0'; ME <= '0'; ready <= '0';
     WHEN 1 => WE <= '1'; ME <= '1'; ready <= '0';
     WHEN 2 => WE <= '0'; ME <= '1'; ready <= '0';
     WHEN 3 => WE <= '0'; ME <= '0'; ready <= '0';
     WHEN 4 => WE <= '0'; ME <= '0'; ready <= '1';
   END CASE;
 END PROCESS output_state;

END data_flow;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_bus_interface_2 IS END test_bus_interface_2;

ARCHITECTURE test OF test_bus_interface_2 IS

COMPONENT bus_interface_2 IS
PORT (
    address: IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    clk, reset, write_slave, ADS, done: IN STD_LOGIC;
    slave_address: OUT STD_LOGIC_VECTOR(29 DOWNTO 0);
    WE, ME, ready: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL address: STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL clk: STD_LOGIC := '1';
SIGNAL reset, write_slave, ADS, WE, ME, done, ready: STD_LOGIC;
SIGNAL slave_address: STD_LOGIC_VECTOR(29 DOWNTO 0);

BEGIN

dut: bus_interface_2 
PORT MAP (address => address, clk => clk,  reset => reset,  write_slave => write_slave,
ADS => ADS, slave_address => slave_address, WE => WE, ME => ME, done => done, ready => ready);

clk <= NOT(clk) AFTER 50 NS;

PROCESS
BEGIN
address <= x"00000000"; write_slave <= '0'; ADS <= '0';
done <= '0';
reset <= '1'; 
WAIT FOR 50 NS; 
reset <= '0'; 
WAIT FOR 350 NS;
-------------------
address <= x"9c2b1234"; write_slave <= '1'; ADS <= '1';
WAIT FOR 100 NS;
ADS <= '0';
WAIT FOR 800 NS;
done <= '1';
WAIT FOR 100 NS;
done <= '0';
WAIT FOR 400 NS;
--------------------
address <= x"77d44321"; write_slave <= '0'; ADS <= '1';
WAIT FOR 100 NS;
ADS <= '0';
WAIT FOR 1200 NS;
--------------------
address <= x"a7d489ab"; write_slave <= '0'; ADS <= '1';
WAIT FOR 100 NS;
ADS <= '0';
WAIT FOR 800 NS;
done <= '1';
WAIT FOR 100 NS;
done <= '0';
WAIT FOR 400 NS;
--------------------
WAIT;
END PROCESS;
END test;
 




