----------------------------------------------------------------------------


----------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY bus_interface_3 IS
PORT (
    Address_Data: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    clk, reset, WriteReq, ReadReq, Ack, done: IN STD_LOGIC;
    Ack_out, Ready, start, oe: OUT STD_LOGIC
);
END bus_interface_3;

ARCHITECTURE data_flow OF bus_interface_3 IS

   TYPE state IS RANGE 0 TO 8;
   SIGNAL current_state: state;


BEGIN

next_state: PROCESS(reset, clk)
BEGIN
  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'event and clk = '1' THEN
    CASE current_state IS
        WHEN 0 => IF (WriteReq AND NOT(Address_Data(7)) AND NOT(Address_Data(6)) 
                               AND Address_Data(5) AND Address_Data(4)) = '1'  
                  THEN current_state <= 1;
                  ELSIF (ReadReq AND NOT(Address_Data(7)) AND NOT(Address_Data(6)) 
                                 AND Address_Data(5) AND Address_Data(4)) = '1' 
                  THEN current_state <= 5; END IF;
        WHEN 1 => current_state <= 2;
        WHEN 2 => current_state <= 3;
        WHEN 3 => IF Ack = '1' THEN current_state <= 4; END IF;
        WHEN 4 => current_state <= 0;
        WHEN 5 => current_state <= 6;
        WHEN 6 => IF done = '1' THEN current_state <= 7; END IF;
        WHEN 7 => IF Ack = '1' THEN current_state <= 8; END IF;
        WHEN 8 => current_state <= 0;
      END CASE;
    END IF;
  END PROCESS next_state;

output_state: PROCESS(current_state)
 BEGIN
   CASE current_state IS
     WHEN 0 => Ack_out <= '0'; Ready <= '0'; start <= '0'; oe <= '0';
     WHEN 1 => Ack_out <= '1'; Ready <= '0'; start <= '0'; oe <= '0';
     WHEN 2 => Ack_out <= '0'; Ready <= '0'; start <= '0'; oe <= '0';
     WHEN 3 => Ack_out <= '0'; Ready <= '1'; start <= '0'; oe <= '0';
     WHEN 4 => Ack_out <= '0'; Ready <= '0'; start <= '1'; oe <= '0';
     WHEN 5 => Ack_out <= '1'; Ready <= '0'; start <= '0'; oe <= '0';
     WHEN 6 => Ack_out <= '0'; Ready <= '0'; start <= '0'; oe <= '0';
     WHEN 7 => Ack_out <= '0'; Ready <= '1'; start <= '0'; oe <= '1';
     WHEN 8 => Ack_out <= '0'; Ready <= '0'; start <= '0'; oe <= '0';
   END CASE;
 END PROCESS output_state;

END data_flow;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_bus_interface_3 IS END test_bus_interface_3;

ARCHITECTURE test OF test_bus_interface_3 IS

COMPONENT bus_interface_3 IS
PORT (
    Address_Data: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    clk, reset, WriteReq, ReadReq, Ack, done: IN STD_LOGIC;
    Ack_out, Ready, start, oe: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL Address_Data: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL clk: STD_LOGIC := '1';
SIGNAL reset, WriteReq, ReadReq, Ack, done, Ack_out, Ready, start, oe: STD_LOGIC;

BEGIN

dut: bus_interface_3 
PORT MAP (Address_Data => Address_Data, clk => clk,  reset => reset,  WriteReq => WriteReq,
ReadReq => ReadReq, Ack => Ack, Ack_out => Ack_out, Ready => Ready, done => done, start => start, oe => oe);

clk <= NOT(clk) AFTER 50 NS;

interface_inputs: PROCESS
BEGIN
Address_Data <= x"00"; WriteReq <= '0'; ReadReq <= '0'; Ack <= '0'; done <= '1';
reset <= '1'; WAIT FOR 100 NS;
reset <= '0'; WAIT FOR 100 NS;
WAIT FOR 200 NS;
--------------------------------
Address_Data <= x"30"; WriteReq <= '1'; 
WAIT UNTIL Ack_out <= '1';
WAIT FOR 100 NS;
WriteReq <= '0';
WAIT UNTIL Ready = '1';
WAIT FOR 100 NS;
Ack <= '1'; 
WAIT FOR 100 NS;
Ack <= '0'; done <= '0';
WAIT FOR 100 NS;
-----------------------------------
ReadReq <= '1'; 
WAIT UNTIL Ack_out <= '1';
WAIT FOR 100 NS;
ReadReq <= '0';
WAIT FOR 500 NS;
done <= '1';
WAIT UNTIL Ready = '1';
WAIT FOR 100 NS;
Ack <= '1';
WAIT FOR 100 NS;
Ack <= '0';
WAIT FOR 100 NS;
------------------------------------
WAIT;
END PROCESS;
END test;
 




