LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.coprocessor_parameters.ALL;
ENTITY cpu IS
PORT (
    data_in: IN SIGNED(p-1 DOWNTO 0);
    clk, Grant: IN STD_LOGIC;
    data_out: OUT SIGNED(p-1 DOWNTO 0);
    address_out: OUT STD_LOGIC_VECTOR(s-1 DOWNTO 0); 
    ADS_out, write_out, Req: OUT STD_LOGIC
);
END cpu;

ARCHITECTURE data_flow OF cpu IS

  PROCEDURE sync IS BEGIN WAIT UNTIL clk'EVENT AND clk = '1'; END sync;

BEGIN

PROCESS

  VARIABLE pixel_address: INTEGER;
  VARIABLE pixel_value: INTEGER;

BEGIN

--initial state: bus request signal inactive and
--outputs to the bus in high impedance
     address_out <= (OTHERS => 'Z'); data_out <= (OTHERS => 'Z'); ADS_out <= 'Z'; write_out <= 'Z'; Req <= '0'; sync;
--request bus control
     Req <= '1'; sync;
     WAIT UNTIL Grant = '1'; sync;
--send unfiltered image to data memory
        FOR i in 0 TO m-1 LOOP
           FOR j in 0 TO n-1 LOOP
              pixel_address := i*n + j;
              --write cycle (as bus master)
                 ADS_out <= '1';  write_out <= '1'; 
                 address_out <= "10"&x"0a" + CONV_STD_LOGIC_VECTOR(pixel_address, s-1);
                 pixel_value := unfiltered(pixel_address);
                 data_out <= CONV_SIGNED(pixel_value, p); sync;
                 ADS_out <= '0'; sync;

                 END LOOP;
              END LOOP;
--send addresses ?a? and ?b? and order ?start? to the 
--coprocessor; three write cycles (as bus master)
     ADS_out <= '1';  write_out <= '1'; 
     address_out <= "00"&x"00";  --"000"&(OTHERS => '0');
     data_out <= x"0a"; sync;
     ADS_out <= '0'; sync; sync;

     ADS_out <= '1';  write_out <= '1'; 
     address_out <= "00"&x"80";  --"001"&(OTHERS => '0');
     data_out <= x"0b"; sync;
     ADS_out <= '0'; sync; sync;
     
     ADS_out <= '1';  write_out <= '1'; 
     address_out <= "01"&x"00";  --"010"&(OTHERS => '0');
     sync;
     ADS_out <= '0'; sync; sync;

--release bus control: bus request signal inactive and
--outputs to the bus in high impedance
     address_out <= (OTHERS => 'Z'); data_out <= (OTHERS => 'Z'); ADS_out <= 'Z'; write_out <= 'Z'; Req <= '0'; sync;

--wait a few cycles while the arbiter grants the bus control --to the coprocessor;
     FOR i IN 0 TO 100 LOOP sync; END LOOP; 

--request bus control; control will be granted when the 
--coprocessor completes its operations
    Req <= '1'; sync;
     WAIT UNTIL Grant = '1'; sync;

--read filtered image from the data memory
        FOR i in 1 TO m-2 LOOP
           FOR j in 1 TO n-2 LOOP
              pixel_address := i*n + j;
                 --read cycle (as bus master)
                 ADS_out <= '1';  write_out <= '0'; 
                 address_out <= "11"&x"0b" + CONV_STD_LOGIC_VECTOR(pixel_address, s-1); sync;
                 ADS_out <= '0'; sync;
                 END LOOP;
              END LOOP;
     --end of operations

WAIT;

END PROCESS;
END data_flow;



