LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY step1 IS
  GENERIC(M: NATURAL);
PORT(
  x1, x2, y1, y2: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  start_step1: IN STD_LOGIC;
  step1_done: OUT STD_LOGIC;
  squ12: OUT STD_LOGIC_VECTOR(2*M-1 DOWNTO 0);
  squ11: INOUT STD_LOGIC_VECTOR(2*M-1 DOWNTO 0)
);
END step1;

ARCHITECTURE circuit OF step1 IS

COMPONENT shift_and_add_multiplier IS
  GENERIC(n: NATURAL:= 64; m: NATURAL:= 64);
PORT(
  x, u: IN STD_LOGIC_VECTOR(n-1 DOWNTO 0);
  y, v: IN STD_LOGIC_VECTOR(m-1 DOWNTO 0);
  clk, reset, start: IN STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(n+m-1 DOWNTO 0);
  done: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL sel, start1, en, done1: STD_LOGIC;
SIGNAL dif11, dif12, mult1: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
SIGNAL a_minus_b: STD_LOGIC_VECTOR(M DOWNTO 0);
SIGNAL b_minus_a: STD_LOGIC_VECTOR(M-1 DOWNTO 0); 
CONSTANT zero: STD_LOGIC_VECTOR(M-1 DOWNTO 0) := CONV_STD_LOGIC_VECTOR(0, M);

CONSTANT N: NATURAL := 7;
SUBTYPE state IS NATURAL RANGE 0 TO N-1;
SIGNAL current_state: state;

BEGIN

WITH sel SELECT dif11 <= x1 WHEN '0', y1 WHEN OTHERS;
WITH sel SELECT dif12 <= x2 WHEN '0', y2 WHEN OTHERS;

--differences:
a_minus_b <= ('0'&dif11) - ('0'&dif12);
b_minus_a <= dif12 - dif11;
WITH a_minus_b(M) SELECT mult1 <= a_minus_b(M-1 DOWNTO 0) WHEN '0', b_minus_a WHEN OTHERS;

first_multiplier: shift_and_add_multiplier
  GENERIC MAP(n => M, m => M)
  PORT MAP(x => mult1, u => zero, y => mult1, v => zero, 
  start => start1, reset => reset, clk => clk, z => squ11, done => done1);

parallel_register: PROCESS(clk)
BEGIN
  IF clk'EVENT AND clk = '1' THEN 
    IF en = '1' THEN squ12 <= squ11; END IF;
  END IF;
END PROCESS;

--control unit
next_state: PROCESS(clk, reset)
BEGIN
  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'EVENT AND clk = '1' THEN 
    CASE current_state IS 
       WHEN 0 => IF start_step1 = '0' THEN current_state <= 1; END IF;
       WHEN 1 => IF start_step1 = '1' THEN current_state <= 2; END IF;
       WHEN 2 => current_state <= 3;
       WHEN 3 => IF done1 = '1' THEN current_state <= 4; END IF;
       WHEN 4 => current_state <= 5;
       WHEN 5 => current_state <= 6;
       WHEN 6 => IF done1 = '1' THEN current_state <= 0; END IF;
     END CASE;
  END IF;
END PROCESS; 

output_function: PROCESS(current_state)
BEGIN
  CASE current_state IS
    WHEN 0 => sel <= '0'; start1 <= '0'; en <= '0'; step1_done <= '1';
    WHEN 1 => sel <= '0'; start1 <= '0'; en <= '0'; step1_done <= '1';
    WHEN 2 => sel <= '0'; start1 <= '1'; en <= '0'; step1_done <= '0';
    WHEN 3 => sel <= '0'; start1 <= '0'; en <= '0'; step1_done <= '0';
    WHEN 4 => sel <= '0'; start1 <= '0'; en <= '1'; step1_done <= '0';
    WHEN 5 => sel <= '1'; start1 <= '1'; en <= '0'; step1_done <= '0';
    WHEN 6 => sel <= '1'; start1 <= '0'; en <= '0'; step1_done <= '0';
  END CASE;
END PROCESS;
END circuit;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY step2 IS
  GENERIC(M: NATURAL);
PORT(
  squ12d, squ11d: IN STD_LOGIC_VECTOR(2*M-1 DOWNTO 0);
  dif21, dif22: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  start_step2: IN STD_LOGIC;
  step2_done: OUT STD_LOGIC;
  root: OUT STD_LOGIC_VECTOR(M DOWNTO 0)
);
END step2;

ARCHITECTURE circuit OF step2 IS

COMPONENT shift_and_add_multiplier IS
  GENERIC(n: NATURAL:= 64; m: NATURAL:= 64);
PORT(
  x, u: IN STD_LOGIC_VECTOR(n-1 DOWNTO 0);
  y, v: IN STD_LOGIC_VECTOR(m-1 DOWNTO 0);
  clk, reset, start: IN STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(n+m-1 DOWNTO 0);
  done: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT SquareRoot IS
  GENERIC(n: NATURAL:= 32);
PORT(
  x: IN STD_LOGIC_VECTOR(2*n-1 DOWNTO 0);
  clk, reset, start:IN STD_LOGIC;
  root: OUT STD_LOGIC_VECTOR(n-1 DOWNTO 0);
  remainder: OUT STD_LOGIC_VECTOR(n DOWNTO 0);
  done: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL start21, start22, done21, done22: STD_LOGIC;
SIGNAL mult2: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
SIGNAL squ2: STD_LOGIC_VECTOR(2*M-1 DOWNTO 0);
SIGNAL sum: STD_LOGIC_VECTOR(2*M+1 DOWNTO 0);
SIGNAL c_minus_d: STD_LOGIC_VECTOR(M DOWNTO 0);
SIGNAL d_minus_c: STD_LOGIC_VECTOR(M-1 DOWNTO 0); 
CONSTANT zero: STD_LOGIC_VECTOR(M-1 DOWNTO 0) := CONV_STD_LOGIC_VECTOR(0, M);

CONSTANT N: NATURAL := 6;
SUBTYPE state IS NATURAL RANGE 0 TO N-1;
SIGNAL current_state: state;

BEGIN

--differences:
c_minus_d <= ('0'&dif21) - ('0'&dif22);
d_minus_c <= dif22 - dif21;
WITH c_minus_d(M) SELECT mult2 <= c_minus_d(M-1 DOWNTO 0) WHEN '0', d_minus_c WHEN OTHERS;

second_multiplier: shift_and_add_multiplier
  GENERIC MAP(n => M, m => M)
  PORT MAP(x => mult2, u => zero, y => mult2, v => zero, 
  start => start21, reset => reset, clk => clk, z => squ2, done => done21);

--adder
sum <= (("00"&squ12d) + ("00"&squ11d)) + ("00"&squ2);

square_root: SquareRoot
GENERIC MAP(n => M+1)
PORT MAP(x => sum, clk => clk, start => start22, reset => reset, root => root, done => done22);

--control unit
next_state: PROCESS(clk, reset)
BEGIN
  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'EVENT AND clk = '1' THEN 
    CASE current_state IS 
       WHEN 0 => IF start_step2 = '0' THEN current_state <= 1; END IF;
       WHEN 1 => IF start_step2 = '1' THEN current_state <= 2; END IF;
       WHEN 2 => current_state <= 3;
       WHEN 3 => IF done21 = '1' THEN current_state <= 4; END IF;
       WHEN 4 => current_state <= 5;
       WHEN 5 => IF done22 = '1' THEN current_state <= 0; END IF;
     END CASE;
  END IF;
END PROCESS; 

output_function: PROCESS(current_state)
BEGIN
  CASE current_state IS
    WHEN 0 => start21 <= '0'; start22 <= '0'; step2_done <= '1';
    WHEN 1 => start21 <= '0'; start22 <= '0'; step2_done <= '1';
    WHEN 2 => start21 <= '1'; start22 <= '0'; step2_done <= '0';
    WHEN 3 => start21 <= '0'; start22 <= '0'; step2_done <= '0';
    WHEN 4 => start21 <= '0'; start22 <= '1'; step2_done <= '0';
    WHEN 5 => start21 <= '0'; start22 <= '0'; step2_done <= '0';
  END CASE;
END PROCESS;

END circuit;

--
------------------------------------------------------------------------------
---- Computes: distance = [(x1-x2)^2 + (y1-y2)^2 + (z1-z2)^2]^0.5
---- x1, x2, y1, y2, z1, z2: M bits
---- distance: M+1 bits
------------------------------------------------------------------------------
--
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY distance_computationST IS
  GENERIC(M: NATURAL);
PORT(
  x1, x2, y1, y2, z1, z2: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  req_in0, ack_in2: IN STD_LOGIC;
  ack_out0, req_out2: OUT STD_LOGIC;
  distance: OUT STD_LOGIC_VECTOR(M DOWNTO 0)
);
END distance_computationST;

ARCHITECTURE circuit OF distance_computationST IS

COMPONENT step1 IS
  GENERIC(M: NATURAL);
PORT(
  x1, x2, y1, y2: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  start_step1: IN STD_LOGIC;
  step1_done: OUT STD_LOGIC;
  squ12: OUT STD_LOGIC_VECTOR(2*M-1 DOWNTO 0);
  squ11: INOUT STD_LOGIC_VECTOR(2*M-1 DOWNTO 0)
);
END COMPONENT;

COMPONENT step2 IS
  GENERIC(M: NATURAL);
PORT(
  squ12d, squ11d: IN STD_LOGIC_VECTOR(2*M-1 DOWNTO 0);
  dif21, dif22: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  start_step2: IN STD_LOGIC;
  step2_done: OUT STD_LOGIC;
  root: OUT STD_LOGIC_VECTOR(M DOWNTO 0)
);
END COMPONENT;

COMPONENT protocol IS
PORT (
  clk, reset, req_in, ack_in, done: IN STD_LOGIC;
  req_out, ack_out, ce, start: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL x1d, x2d, y1d, y2d, z1d, z2d, dif21, dif22: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
SIGNAL ce0, start_step1, step1_done, ce1, start_step2, step2_done, ce2: STD_LOGIC;
SIGNAL squ12, squ12d, squ11, squ11d: STD_LOGIC_VECTOR(2*M-1 DOWNTO 0);
SIGNAL req_in1, ack_out1, req_in2, ack_out2: STD_LOGIC; 
SIGNAL root: STD_LOGIC_VECTOR(M DOWNTO 0);
PROCEDURE sync IS
BEGIN
  WAIT UNTIL clk'EVENT AND clk = '1';
END sync;


BEGIN

first_step: step1 
   GENERIC MAP (M => M)
   PORT MAP(x1 => x1d, x2 => x2d, y1 => y1d, y2 => y2d, clk => clk, reset => reset, 
   start_step1 => start_step1, step1_done => step1_done, squ12 => squ12, squ11 => squ11); 

second_step: step2 
   GENERIC MAP (M => M)
   PORT MAP(squ12d => squ12d, squ11d => squ11d, dif21 => dif21, dif22 => dif22, 
   clk => clk, reset => reset, start_step2 => start_step2, step2_done => step2_done, root => root);

protocol_step1: protocol
   PORT MAP (clk => clk, reset => reset, req_in => req_in0, ack_in => ack_out1,
   done => step1_done, req_out => req_in1, ack_out => ack_out0, ce => ce0, start => start_step1);  

protocol_step2: protocol
   PORT MAP (clk => clk, reset => reset, req_in => req_in1, ack_in => ack_out2,
   done => step2_done, req_out => req_in2, ack_out => ack_out1, ce => ce1, start => start_step2);  

--protocol_step3: protocol
--   PORT MAP (clk => clk, reset => reset, req_in => req_in2, ack_in => ack_in2,
--   done => '1', req_out => req_out2, ack_out => ack_out2, ce => ce2);  


protocol_step3: PROCESS
BEGIN
 ce2 <= '0'; req_out2 <= '0'; ack_out2 <= '0'; sync;
  LOOP
    IF reset = '1' THEN EXIT; END IF;
    IF req_in2 = '1'THEN
      ce2 <= '1'; sync;
      ce2 <= '0'; ack_out2 <= '1'; sync; 
      ack_out2 <= '0'; sync;
      req_out2 <= '1';
      WAIT UNTIL ack_in2 = '1';
      req_out2 <= '0'; sync;
    ELSE sync;
   END IF;  
  END LOOP;
END PROCESS;
  


--
input_register: PROCESS(clk)
BEGIN 
  IF clk'event and clk = '1' THEN 
    IF ce0 = '1' THEN 
     x1d <= x1; y1d <= y1; x2d <= x2; y2d <= y2; z1d <= z1; z2d <= z2; 
    END IF;
  END IF;
END PROCESS; 

pipeline_register: PROCESS(clk)
BEGIN 
  IF clk'event and clk = '1' THEN 
    IF ce1 = '1' THEN 
      squ12d <= squ12; squ11d <= squ11; dif21 <= z1d; dif22 <= z2d; 
    END IF;
  END IF;
END PROCESS; 

output_register: PROCESS(clk)
BEGIN 
  IF clk'event and clk = '1' THEN 
    IF ce2 = '1' THEN 
     distance <= root; 
    END IF;
  END IF;
END PROCESS; 

END circuit;



LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_distance_computationST IS END test_distance_computationST;

ARCHITECTURE test OF test_distance_computationST IS

COMPONENT distance_computationST IS
  GENERIC(M: NATURAL);
PORT(
  x1, x2, y1, y2, z1, z2: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  req_in0, ack_in2: IN STD_LOGIC;
  ack_out0, req_out2: OUT STD_LOGIC;
  distance: OUT STD_LOGIC_VECTOR(M DOWNTO 0)
);
END COMPONENT;

SIGNAL x1, x2, y1, y2, z1, z2: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL reset: STD_LOGIC;
SIGNAL clk: STD_LOGIC := '0';
SIGNAL req_in0, ack_out0, req_out2, ack_in2: STD_LOGIC;
SIGNAL distance: STD_LOGIC_VECTOR(8 DOWNTO 0); 

PROCEDURE sync IS
BEGIN
  WAIT UNTIL clk'EVENT AND clk = '1';
END sync;


BEGIN

dut: distance_computationST
GENERIC MAP(M => 8)
PORT MAP(x1 => x1, x2 => x2, y1 => y1, y2 => y2, z1 => z1, z2 => z2,
clk => clk, reset => reset, req_in0 => req_in0, ack_out0 => ack_out0, 
req_out2 => req_out2, ack_in2 => ack_in2, distance => distance);

clk <= NOT(clk) AFTER 50 NS;

test_vectors: PROCESS
BEGIN
reset <= '1'; 
x1 <= "00000011"; y1 <= "00000010"; z1 <= "00000001"; 
x2 <= "00000111"; y2 <= "00000110"; z2 <= "00000101"; 
req_in0 <= '0'; 
sync;
reset <= '0'; 
sync;

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00001011"; y1 <= "00001010"; z1 <= "00001001"; 
x2 <= "00010111"; y2 <= "00000110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00000011"; y1 <= "00010010"; z1 <= "00000001"; 
x2 <= "00000111"; y2 <= "00000110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00001011"; y1 <= "00001010"; z1 <= "00001001"; 
x2 <= "00000111"; y2 <= "00100110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00000000"; y1 <= "00000000"; z1 <= "00000000"; 
x2 <= "11111111"; y2 <= "11111111"; z2 <= "11111111"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00000011"; y1 <= "00010010"; z1 <= "00000001"; 
x2 <= "00000111"; y2 <= "00000110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00001011"; y1 <= "00001010"; z1 <= "00001001"; 
x2 <= "00000111"; y2 <= "00100110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00000000"; y1 <= "00000000"; z1 <= "00000000"; 
x2 <= "11111111"; y2 <= "11111111"; z2 <= "11111111"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00000011"; y1 <= "00010010"; z1 <= "00000001"; 
x2 <= "00000111"; y2 <= "00000110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00001011"; y1 <= "00001010"; z1 <= "00001001"; 
x2 <= "00000111"; y2 <= "00100110"; z2 <= "00000101"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
sync;
x1 <= "00000000"; y1 <= "00000000"; z1 <= "00000000"; 
x2 <= "11111111"; y2 <= "11111111"; z2 <= "11111111"; 

req_in0 <= '1';
WAIT UNTIL ack_out0 = '1';
req_in0 <= '0';
WAIT;
END PROCESS;

output_block: PROCESS
BEGIN
  ack_in2 <= '0'; 
  LOOP
    IF req_out2 = '1'THEN
      ack_in2 <= '1'; sync; 
      ack_in2 <= '0'; sync;
    ELSE sync;
    END IF;  
  END LOOP;
END PROCESS;


END test;