library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_signed.all;
package pid_parameters is
  constant m: integer := 16; --number of bits of memory word
  constant n: integer := 16;  --number of memory words
  constant p: integer := 4;  --number of address bits
  constant k11: signed(m-1 downto 0) := x"006b";
  constant k21: signed(m-1 downto 0) := x"ff98";
  constant k31: signed(m-1 downto 0) := x"0002";
--  constant k12: signed(m-1 downto 0) := x"006b";
--  constant k22: signed(m-1 downto 0) := x"ff98";
--  constant k32: signed(m-1 downto 0) := x"0002";
  constant k12: signed(m-1 downto 0) := x"0049";
  constant k22: signed(m-1 downto 0) := x"ffaa";
  constant k32: signed(m-1 downto 0) := x"00c9";
  constant one: signed(m-1 downto 0) := x"0001"; 
  constant sample_period: integer := 32; --number of clock cycles 
end pid_parameters;

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.pid_parameters.all;
entity three_port_register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write_reg, reset: in std_logic;
  address_A, address_B, address_W: in std_logic_vector(p-1 downto 0);
  data_out_A, data_out_B: out std_logic_vector(m-1 downto 0)  
);
end three_port_register_file;

architecture structure of three_port_register_file is

type memory is array (0 to n-1) of std_logic_vector(m-1 downto 0);
signal X: memory;
signal EN: std_logic_vector(0 to n-1);

begin

  decoder: process(address_W, write_reg)
  begin
    for i in 0 to n-1 loop
      if i = conv_integer(address_W) then EN(i) <= write_reg;
      else EN(i) <= '0';
      end if;
    end loop;
  end process;
  
   bank_registers: process(clk, reset)
   begin
     if reset = '1' THEN
       for i in 0 to n-1 loop X(i) <= (OTHERS => '0'); end loop;
     elsif clk'event and clk = '1' then
       for i in 0 to n-1 loop
         if EN(i) = '1' then X(i) <= data_in; end if;
       end loop;
     end if;
    end process;

  multiplexer_A: process(address_A, X)
  begin
     for i in 0 to n-1 loop
        if i = conv_integer(address_A) then data_out_A <= X(i); end if;
     end loop;
   end process;  

  multiplexer_B: process(address_B, X)
  begin
     for i in 0 to n-1 loop
        if i = conv_integer(address_B) then data_out_B <= X(i); end if;
     end loop;
   end process;  


end structure;	

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.pid_parameters.all;
entity computation_resource is
port (
  x: in signed(m-1 downto 0);
  y: in signed(m-1 downto 0);
  oper: in std_logic_vector(2 downto 0);
  z: out signed(m-1 downto 0)
);
end computation_resource;

architecture behavior of computation_resource is

signal long_z, long_x: signed(2*m-1 downto 0);

begin

bit_extension: PROCESS(x)
BEGIN
   FOR i IN m TO 2*m-1 LOOP long_x(i) <= x(m-1); END LOOP;
   FOR i IN 0 TO m-1 LOOP long_x(i) <= x(i); END LOOP;
END PROCESS;

   with oper select long_z <=  long_x - one*y when "001", 
                          long_x + k11*y when "010",
                          long_x + k21*y when "100",
                          long_x + k31*y when "110",
                          long_x + k12*y when "011",
                          long_x + k22*y when "101",
                          long_x + k32*y when "111",
                          long_x when others;

   z <= long_z(m-1 downto 0);

end behavior;

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.pid_parameters.all;
entity double_pid_controller2 is
port (
  reference1, s1, reference2, s2: in signed(m-1 downto 0);
  clk, reset: in std_logic;
  u1, u2: out signed(m-1 downto 0)
);
end double_pid_controller2;

architecture circuit of double_pid_controller2 is

component computation_resource is
port (
  x: in signed(m-1 downto 0);
  y: in signed(m-1 downto 0);
  oper: in std_logic_vector(2 downto 0);
  z: out signed(m-1 downto 0)
);
end component;

component three_port_register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write_reg, reset: in std_logic;
  address_A, address_B, address_W: in std_logic_vector(p-1 downto 0);
  data_out_A, data_out_B: out std_logic_vector(m-1 downto 0)  
);
end component;

signal x, z: signed(m-1 downto 0);
signal y: signed(m-1 downto 0);
signal data_out_A, data_out_B: std_logic_vector(m-1 downto 0);
signal sel, sel1, sel2: std_logic_vector(1 downto 0);
signal out_en1, out_en2, write_reg, write_reg1, write_reg2, time_out: std_logic;
signal oper, oper1, oper2: std_logic_vector(2 downto 0);
signal address_A, address_B, address_W, address_A1, address_B1, address_W1, 
       address_A2, address_B2, address_W2 : std_logic_vector(p-1 downto 0);
signal count: natural;
--signal long_reference: signed(m-1 downto 0);
signal signed_data_out_A, signed_data_out_B: signed(m-1 downto 0);
signal unsigned_z: std_logic_vector(m-1 downto 0);

SUBTYPE state IS NATURAL RANGE 0 TO 4;
SIGNAL current_state: state;
SUBTYPE state1 IS NATURAL RANGE 0 TO 7;
SIGNAL current_state1: state1;
SUBTYPE state2 IS NATURAL RANGE 0 TO 7;
SIGNAL current_state2: state2;

SIGNAL start1, start2, done1, done2: std_logic;

begin

sel <= sel1 OR sel2;
write_reg <= write_reg1 OR write_reg2;
oper <= oper1 OR oper2;
address_A <= address_A1 OR address_A2;
address_B <= address_B1 OR address_B2;
address_W <= address_W1 OR address_W2;


---DATA PATH
first_component: computation_resource
port map (x => x, y => y, oper => oper, z => z);

signed_data_out_A <= signed(data_out_A);
signed_data_out_B <= signed(data_out_B);
unsigned_z <= std_logic_vector(z);

second_component: three_port_register_file
port map (data_in => unsigned_z, address_A => address_A,  address_B => address_B, address_W => address_W, 
write_reg => write_reg, clk => clk, reset => reset, data_out_A => data_out_A,  data_out_B => data_out_B);

WITH sel SELECT x <= reference1 WHEN "00", reference2 WHEN "01", signed_data_out_A WHEN OTHERS;
WITH sel SELECT y <= s1 WHEN "00", s2 WHEN "01", signed_data_out_B WHEN OTHERS;

output_register1: process(clk)
BEGIN
   IF clk'event AND clk = '1' THEN
     IF out_en1 = '1' THEN u1 <= z; END IF;
   END IF;
END PROCESS;

output_register2: process(clk)
BEGIN
   IF clk'event AND clk = '1' THEN
     IF out_en2 = '1' THEN u2 <= z; END IF;
   END IF;
END PROCESS;

---TIMER
counter: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN count <= sample_period - 1;
   ELSIF clk'event AND clk = '1' THEN count <= (count - 1) MOD sample_period;
   END IF;
END PROCESS;
WITH count SELECT time_out <= '1' WHEN 0, '0' WHEN OTHERS;


--MAIN CONTROL UNIT
main_next_state_function: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state IS
         WHEN 0 => current_state <= 1;
         WHEN 1 => IF done1 = '1' THEN current_state <= 2; END IF;
         WHEN 2 => current_state <= 3;
         WHEN 3 => IF done2 = '1' THEN current_state <= 4; END IF;
         WHEN OTHERS => IF time_out = '1' THEN current_state <= 0; END IF;
      END CASE;
   END IF;
END PROCESS;

main_output_function: PROCESS(current_state)
BEGIN
      CASE current_state IS
         WHEN 0 => start1 <= '1'; start2 <= '0';
         WHEN 1 => start1 <= '0'; start2 <= '0';
         WHEN 2 => start1 <= '0'; start2 <= '1';
         WHEN 3 => start1 <= '0'; start2 <= '0';
         WHEN OTHERS => start1 <= '0'; start2 <= '0';
      END CASE;
END PROCESS;

--PID1 CONTROL UNIT
next_state_function1: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state1 <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state1 IS
         WHEN 0  => IF start1 = '1' THEN current_state1 <= 1; END IF;
         WHEN 1 to 6 => current_state1 <= current_state1 + 1;
         WHEN 7 => current_state1 <= 0;
      END CASE;
   END IF;
END PROCESS;

output_function1: PROCESS(current_state1)
BEGIN
      CASE current_state1 IS
         WHEN 0 => write_reg1 <= '0'; sel1 <= "00"; address_W1 <= "0000"; 
         address_A1 <= "0000"; address_B1 <= "0000"; oper1 <= "000"; out_en1 <= '0'; done1 <= '0';
         WHEN 1 => write_reg1 <= '1'; sel1 <= "00"; address_W1 <= "0000"; 
         address_A1 <= "0000"; address_B1 <= "0000"; oper1 <= "001"; out_en1 <= '0'; done1 <= '0'; 
         WHEN 2 => write_reg1 <= '1'; sel1 <= "11"; address_W1 <= "1111"; 
         address_A1 <= "0011"; address_B1 <= "0000"; oper1 <= "010"; out_en1 <= '0'; done1 <= '0'; 
         WHEN 3 => write_reg1 <= '1'; sel1 <= "11"; address_W1 <= "1111"; 
         address_A1 <= "1111"; address_B1 <= "0001"; oper1 <= "100"; out_en1 <= '0'; done1 <= '0'; 
         WHEN 4 => write_reg1 <= '1'; sel1 <= "11"; address_W1 <= "0011"; 
         address_A1 <= "1111"; address_B1 <= "0010"; oper1 <= "110"; out_en1 <= '1'; done1 <= '0'; 
         WHEN 5 => write_reg1 <= '1'; sel1 <= "11"; address_W1 <= "0010"; 
         address_A1 <= "0001"; address_B1 <= "0000"; oper1 <= "000"; out_en1 <= '0'; done1 <= '0'; 
         WHEN 6 => write_reg1 <= '1'; sel1 <= "11"; address_W1 <= "0001"; 
         address_A1 <= "0000"; address_B1 <= "0000"; oper1 <= "000"; out_en1 <= '0'; done1 <= '0'; 
         WHEN OTHERS => write_reg1 <= '0'; sel1 <= "00"; address_W1 <= "0000"; 
         address_A1 <= "0000"; address_B1 <= "0000"; oper1 <= "000"; out_en1 <= '0'; done1 <= '1';
      END CASE;
END PROCESS;

--PID2 CONTROL UNIT
next_state_function2: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state2 <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state2 IS
         WHEN 0  => IF start2 = '1' THEN current_state2 <= 1; END IF;
         WHEN 1 to 6 => current_state2 <= current_state2 + 1;
         WHEN 7 => current_state2 <= 0;
      END CASE;
   END IF;
END PROCESS;

output_function2: PROCESS(current_state2)
BEGIN
      CASE current_state2 IS
         WHEN 0 => write_reg2 <= '0'; sel2 <= "00"; address_W2 <= "0000"; 
         address_A2 <= "0000"; address_B2 <= "0000"; oper2 <= "000"; out_en2 <= '0'; done2 <= '0';
         WHEN 1 => write_reg2 <= '1'; sel2 <= "01"; address_W2 <= "1000"; 
         address_A2 <= "0000"; address_B2 <= "0000"; oper2 <= "001";  out_en2 <= '0'; done2 <= '0';
         WHEN 2 => write_reg2 <= '1'; sel2 <= "11"; address_W2 <= "1111"; 
         address_A2 <= "1011"; address_B2 <= "1000"; oper2 <= "011";  out_en2 <= '0'; done2 <= '0';
         WHEN 3 => write_reg2 <= '1'; sel2 <= "11"; address_W2 <= "1111"; 
         address_A2 <= "1111"; address_B2 <= "1001"; oper2 <= "101";  out_en2 <= '0'; done2 <= '0';
         WHEN 4 => write_reg2 <= '1'; sel2 <= "11"; address_W2 <= "1011"; 
         address_A2 <= "1111"; address_B2 <= "1010"; oper2 <= "111";  out_en2 <= '1'; done2 <= '0';
         WHEN 5 => write_reg2 <= '1'; sel2 <= "11"; address_W2 <= "1010"; 
         address_A2 <= "1001"; address_B2 <= "0000"; oper2 <= "000";  out_en2 <= '0'; done2 <= '0';
         WHEN 6 => write_reg2 <= '1'; sel2 <= "11"; address_W2 <= "1001"; 
         address_A2 <= "1000"; address_B2 <= "0000"; oper2 <= "000";  out_en2 <= '0'; done2 <= '0';
         WHEN OTHERS => write_reg2 <= '0'; sel2 <= "00"; address_W2 <= "0000"; 
         address_A2 <= "0000"; address_B2 <= "0000"; oper2 <= "000";  out_en2 <= '0'; done2 <= '1';
      END CASE;
END PROCESS;


END circuit;


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.pid_parameters.all;
entity test_double_pid_controller2 IS END test_double_pid_controller2;

architecture test of test_double_pid_controller2 is

component double_pid_controller2 is
port (
  reference1, s1, reference2, s2: in signed(m-1 downto 0);
  clk, reset: in std_logic;
  u1, u2: out signed(m-1 downto 0)
);
end component;

signal reference1, s1, reference2, s2: signed(m-1 downto 0);
signal u1, u2: signed(m-1 downto 0); 
signal reset : STD_LOGIC;
signal clk: STD_LOGIC := '0';

BEGIN

dut: double_pid_controller2
port map(reference1 => reference1, s1 => s1, u1 => u1, 
   reference2 => reference2, s2 => s2, u2 => u2, reset => reset, clk => clk);

clk <= NOT(clk) AFTER 50 NS;

test_vectors: PROCESS
BEGIN
reset <= '1';
reference1 <= x"0007";
reference2 <= x"0007";
s1 <= x"000f";
s2 <= x"001f";
WAIT FOR 100 NS;
reset <= '0';
WAIT FOR 2600 NS;
s1 <= x"0005";
s2 <= x"fff7";
WAIT FOR 3200 NS;
s1 <= x"0008";
s2 <= x"0018";
WAIT FOR 3200 NS;
s1 <= x"0007";
s2 <= x"fffa";
WAIT;
END PROCESS;
END test;