LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
USE IEEE.MATH_REAL.ALL; -- for UNIFORM, TRUNC
ENTITY emitter IS
  GENERIC(key: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset: IN STD_LOGIC;
    master_clk, ready: OUT STD_LOGIC;
    message, data_out: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END emitter;

ARCHITECTURE circuit OF emitter IS

COMPONENT signal_generator IS 
  PORT(
   clk, reset, done: IN STD_LOGIC;
    random: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    start: OUT STD_LOGIC
  );
END COMPONENT;

COMPONENT mod_m_exponentiation IS 
  PORT(
    x, y: IN STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    clk, reset, start: STD_LOGIC;
    z: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    done: OUT STD_LOGIC
  );
END COMPONENT;

SIGNAL random: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL start_exp, exp_done: STD_LOGIC;

BEGIN

first_component: signal_generator
  PORT MAP(clk => clk, reset => reset, done => exp_done,
    random => random, start => start_exp);

second_component: mod_m_exponentiation 
  PORT MAP(x => key, y => random, clk => clk, reset => reset, 
    start => start_exp, z => data_out, done => exp_done);

message <= random;

master_clk <= clk;

ready <= exp_done;

END circuit;



LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
ENTITY test_emitter IS END test_emitter;

ARCHITECTURE test OF test_emitter IS

COMPONENT emitter IS
  GENERIC(key: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset: IN STD_LOGIC;
    master_clk, ready: OUT STD_LOGIC;
    message, data_out: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END COMPONENT;

SIGNAL clk: STD_LOGIC := '0';
SIGNAL master_clk, ready, reset: STD_LOGIC;
SIGNAL message, data_out: STD_LOGIC_VECTOR(k-1 DOWNTO 0);


BEGIN

dut: emitter
  GENERIC MAP(key => x"05")
  PORT MAP(
    clk => clk, reset => reset, message => message, data_out => data_out, 
    master_clk => master_clk, ready => ready); 

clk <= NOT(clk) AFTER 50 NS;

test_vectors: PROCESS
BEGIN
reset <= '1';
WAIT FOR 100 NS;
reset <= '0';
WAIT FOR 100 NS;

WAIT FOR 1200000 NS;

WAIT;

END PROCESS;


END test;
