LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
USE IEEE.MATH_REAL.ALL; -- for UNIFORM, TRUNC
ENTITY encoded_transmission IS
  PORT(
    clk, reset: IN STD_LOGIC;
    message, encoded, decoded: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END encoded_transmission;

ARCHITECTURE circuit OF encoded_transmission IS

COMPONENT emitter IS 
  GENERIC(key: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset: IN STD_LOGIC;
    master_clk, ready: OUT STD_LOGIC;
    message, data_out: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
 );
END COMPONENT;

COMPONENT receiver IS 
  GENERIC(key: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset, ready: IN STD_LOGIC;
    data_in: IN STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    decoded: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
  );
END COMPONENT;

SIGNAL data: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL ready, master_clk: STD_LOGIC;

BEGIN

first_component: emitter
  GENERIC MAP(key => x"05")
  PORT MAP(clk => clk, reset => reset, master_clk => master_clk,
    ready => ready, message => message, data_out => data);

second_component: receiver
  GENERIC MAP(key => x"1d")
  PORT MAP(clk => master_clk, reset => reset, ready => ready, data_in => data, 
     decoded => decoded);

encoded <= data;

END circuit;



LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
ENTITY test_rsa IS END test_rsa;

ARCHITECTURE test OF test_rsa IS

COMPONENT encoded_transmission IS
  PORT(
    clk, reset: IN STD_LOGIC;
    message, encoded, decoded: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END COMPONENT;

SIGNAL clk: STD_LOGIC := '0';
SIGNAL reset: STD_LOGIC;
SIGNAL message, encoded, decoded: STD_LOGIC_VECTOR(k-1 DOWNTO 0);


BEGIN

dut: encoded_transmission
  PORT MAP(
    clk => clk, reset => reset, message => message, encoded => encoded, 
    decoded => decoded); 

clk <= NOT(clk) AFTER 50 NS;

test_vectors: PROCESS
BEGIN
reset <= '1';
WAIT FOR 100 NS;
reset <= '0';
WAIT FOR 100 NS;

WAIT FOR 1200000 NS;

WAIT;

END PROCESS;


END test;
