LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
USE IEEE.MATH_REAL.ALL; -- for UNIFORM, TRUNC
ENTITY encoding_system2 IS
  PORT(
    clk, reset: IN STD_LOGIC;
    error_full, error_empty: OUT STD_LOGIC;    
    s: INOUT std_logic_vector(p-1 downto 0);
    data_out: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END encoding_system2;

ARCHITECTURE circuit OF encoding_system2 IS

COMPONENT signal_generator IS 
  PORT(
    clk, reset, exp_done: IN STD_LOGIC;
    x, y: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    start_exp: OUT STD_LOGIC
  );
END COMPONENT;

COMPONENT mod_m_exponent IS 
  PORT(
    x, y: IN STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    clk, reset, start: STD_LOGIC;
    z: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    done: OUT STD_LOGIC
  );
END COMPONENT;

COMPONENT fifo IS
PORT (
  data_in: in std_logic_vector(k-1 downto 0);
  clk, reset, write, read: in std_logic;
  empty, full: out std_logic;
  s: inout std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(k-1 downto 0)
);  
END COMPONENT;  


SIGNAL x, y, z: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL start_exp, exp_done, full, empty, write_fifo, read_fifo: STD_LOGIC;

TYPE state IS RANGE 0 TO 3;
SIGNAL current_state: state;

CONSTANT output_period: integer := 65000; --number of clock cycles  
SIGNAL count: natural;
SIGNAL time_out: STD_LOGIC;

TYPE output_state IS RANGE 0 TO 2;
SIGNAL current_output_state: output_state;


BEGIN

first_component: signal_generator
  PORT MAP(clk => clk, reset => reset, exp_done => exp_done,
    x => x, y => y, start_exp => start_exp);

second_component: mod_m_exponent 
  PORT MAP(x => x, y => y, clk => clk, reset => reset, 
    start => start_exp, z => z, done => exp_done);

third_component: fifo
  PORT MAP (data_in => z, clk => clk, reset => reset, write => write_fifo, 
    read => read_fifo, empty => empty, full => full, data_out => data_out, s => s);

-- FIFO write
next_state: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state IS
         WHEN 0 => IF exp_done = '0' THEN current_state <= 1; END IF;
         WHEN 1 => IF exp_done = '1' AND full = '0' THEN current_state <= 2; 
                   ELSIF exp_done = '1' AND full = '1' THEN current_state <= 3; END IF;
         WHEN 2 => current_state <= 0;
         WHEN 3 => current_state <= 0;
      END CASE;
   END IF;
END PROCESS;

output_function: PROCESS(current_state)
BEGIN
      CASE current_state IS
         WHEN 0 TO 1 => error_full <= '0'; write_fifo <= '0';
         WHEN 2 => error_full <= '0'; write_fifo <= '1'; 
         WHEN 3 => error_full <= '1'; write_fifo <= '0'; 
      END CASE;
END PROCESS;

-- FIFO read
counter: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN count <= output_period - 1;
   ELSIF clk'event AND clk = '1' THEN count <= (count - 1) MOD output_period;
   END IF;
END PROCESS;
WITH count SELECT time_out <= '1' WHEN 0, '0' WHEN OTHERS;

next_output_state: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_output_state <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_output_state IS
         WHEN 0 => IF time_out = '1' AND empty = '0' THEN current_output_state <= 1; 
                   ELSIF time_out = '1' AND empty = '1' THEN current_output_state <= 2; END IF;
         WHEN 1 => current_output_state <= 0;
         WHEN 2 => current_output_state <= 0;
      END CASE;
   END IF;
END PROCESS;

output_output_function: PROCESS(current_output_state)
BEGIN
      CASE current_output_state IS
         WHEN 0 => error_empty <= '0'; read_fifo <= '0';
         WHEN 1 => error_empty <= '0'; read_fifo <= '1';
         WHEN 2 => error_empty <= '1'; read_fifo <= '0';
      END CASE;
END PROCESS;

END circuit;


LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
ENTITY test_encoding_system2 IS END test_encoding_system2;

ARCHITECTURE test OF test_encoding_system2 IS

COMPONENT encoding_system2 IS
  PORT(
    clk, reset: IN STD_LOGIC;
    error_full, error_empty: OUT STD_LOGIC;    
    s: INOUT std_logic_vector(p-1 downto 0);
    data_out: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END COMPONENT;


SIGNAL clk: STD_LOGIC := '0';
SIGNAL reset, error_full, error_empty: STD_LOGIC;
SIGNAL data_out: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL s: std_logic_vector(p-1 downto 0);


BEGIN

dut: encoding_system2 
  PORT MAP(
    clk => clk, reset => reset, data_out => data_out, 
    error_full => error_full, error_empty => error_empty, s => s);

clk <= NOT(clk) AFTER 50 NS;

reset <= '1', '0' AFTER 100 NS;

END test;