library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
package main_parameters is
  constant m: natural := 8; -- m bits per word
  constant n: natural := 16; -- n words
  constant p: natural := 4; -- p address bits
  constant zero: std_logic_vector(m-1 downto 0) := conv_std_logic_vector(0, m);
end main_parameters;

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity filo is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, reset, write, read: in std_logic;
  empty, full: out std_logic;
  data_out: out std_logic_vector(m-1 downto 0)
);  
end filo;  

architecture structure of filo is

component one_port_register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write, read: in std_logic;
  write_address, read_address: in std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(m-1 downto 0)  
);
end component;

signal write_address, read_address: std_logic_vector(p-1 downto 0);

begin

process (clk, reset, write_address, read_address)
begin
  if reset = '1' then 
     write_address <= (others => '0'); 
     full <= '0'; empty <= '1';
  elsif clk'event and clk = '1' then
     if write = '1' then 
        empty <= '0';
        if write_address = n-1 then full <= '1'; end if;
        write_address <= write_address + 1;
     elsif read = '1' then
        full <= '0'; 
        if write_address = 1 then empty <= '1'; end if;
        write_address <= write_address - 1;
     end if;
   end if;
end process;

read_address <= write_address - 1;

main_component: one_port_register_file
port map (
  data_in => data_in, clk => clk, write => write, read => read,
  write_address => write_address, read_address => read_address, data_out => data_out);

end structure;	


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity test_filo is end test_filo;

architecture test of test_filo is

component filo is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, reset, write, read: in std_logic;
  empty, full: out std_logic;
  data_out: out std_logic_vector(m-1 downto 0)
);  
end component;  

signal data_in, data_out: std_logic_vector(7 downto 0);
signal clk: std_logic := '0';
signal reset, empty, full, write, read: std_logic;

begin
   
dut: filo 
port map( data_in => data_in, data_out => data_out, clk => clk, reset => reset,
empty => empty, full => full, write => write, read => read);

clk <= not clk after 50 ns;

process
begin
   reset <= '1'; write <= '0'; read <= '0';
   wait for 100 ns;
   reset <= '0';
   wait for 100 ns;

   write <= '1';
   data_in <= "00000000";
   wait for 100 ns;
   for i in 1 to 9 loop
      data_in <= data_in + 1;
      wait for 100 ns;
   end loop;
   write <= '0';

   read <= '1';
   wait for 500 ns;
   read <= '0';
   wait for 100 ns;

   write <= '1';
   for i in 1 to 9 loop
      data_in <= data_in + 1;
      wait for 100 ns;
   end loop;
   write <= '0';

   read <= '1';
   wait for 1000 ns;
   read <= '0';
   wait for 100 ns;

   wait;
end process;

end test;   
