----------------------------------------------------------------------------


----------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_SIGNED.ALL;
PACKAGE image_filter_parameters IS
  CONSTANT m: INTEGER := 9; -- m rows
  CONSTANT n: INTEGER := 16; -- n columns
  CONSTANT p: INTEGER := 8; -- 8 bits per pixel
  CONSTANT s: INTEGER := 10; -- address bits

--  TYPE memory IS ARRAY (0 TO m*n - 1) OF INTEGER;
  TYPE memory IS ARRAY (0 TO 2**s - 1) OF INTEGER;

  CONSTANT image: memory := (
  1, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
  1, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
  1, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
  1, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
  1, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
  1, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
  OTHERS => 0
  );


  TYPE mask IS ARRAY (0 TO 8) OF INTEGER;
  CONSTANT c: mask := (
  -1, -1, -1,
  -1,  8, -1,
  -1, -1, -1
  );
END image_filter_parameters;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.image_filter_parameters.ALL;
ENTITY image_filter_DF IS
PORT (
    a, b: IN STD_LOGIC_VECTOR(s-1 DOWNTO 0);
    data_in: IN SIGNED(p-1 DOWNTO 0);
    clk, start, grant: IN STD_LOGIC;
    address: OUT STD_LOGIC_VECTOR(s-1 DOWNTO 0);
    data_out: OUT SIGNED(p-1 DOWNTO 0);
    req, done, we, oe: OUT STD_LOGIC
);
END image_filter_DF;

ARCHITECTURE data_flow OF image_filter_DF IS


PROCEDURE sync IS
BEGIN
  WAIT UNTIL clk'EVENT AND clk = '1';
END sync;

BEGIN

PROCESS
  VARIABLE pixel_address, pixel_value: INTEGER;
  VARIABLE acc: SIGNED(p-1 DOWNTO 0);
BEGIN
  LOOP
     req <= '0'; done <= '0'; address <= (OTHERS => 'Z'); data_out <= (OTHERS => 'Z'); we <= 'Z'; oe <= 'Z';
     WAIT UNTIL start = '1';
     req <= '1';
     WAIT UNTIL grant = '1'; 
     FOR i in 1 TO m-1 LOOP
        FOR j in 1 TO n-1 LOOP
           pixel_address := i*n + j; acc := (OTHERS => '0'); oe <= '1'; we <= '0'; 
           FOR k IN 0 TO 2 LOOP
              FOR l IN 0 TO 2 LOOP
                 address <= CONV_STD_LOGIC_VECTOR(pixel_address + CONV_INTEGER(a) + (k-1)*n + (l-1), s); sync;
                 pixel_value := CONV_INTEGER(data_in); sync;
                 acc := acc + c(k*3 + l)*pixel_value;
              END LOOP;
            END LOOP;
            oe <= '0'; sync;
            address <= CONV_STD_LOGIC_VECTOR(pixel_address + CONV_INTEGER(b), s); data_out <= acc; sync;
            we <= '1'; sync;
            we <= '0'; sync;
            data_out <= (OTHERS => 'Z'); sync;
         END LOOP;
      END LOOP;
      done <= '1'; sync;
  END LOOP;
END PROCESS;
END data_flow;

