--LIBRARY IEEE;
library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
USE WORK.image_filter_parameters.ALL;
entity image_memory is
port (
  data_in: in signed(p-1 downto 0);
  clk, we, oe: in std_logic;
  address: in std_logic_vector(s-1 downto 0);
  data_out: out signed(p-1 downto 0)  
);
end image_memory;

architecture structure of image_memory is

SIGNAL X: memory := image;
signal EN: std_logic_vector(0 to 2**s-1);
SIGNAL internal_data_out: signed(p-1 downto 0);  

begin

  decoder: process(address, we)
  begin
    for i in 0 to 2**s-1 loop
      if i = conv_integer(address) then EN(i) <= we;
      else EN(i) <= '0';
      end if;
    end loop;
  end process;
  
   bank_registers: process(clk)
   begin
     if clk'event and clk = '1' then
       for i in 0 to 2**s-1 loop
         if EN(i) = '1' then X(i) <= CONV_INTEGER(data_in); end if;
       end loop;
     end if;
    end process;

  multiplexer: process(address, X)
  begin
     for i in 0 to 2**s-1 loop
        if i = conv_integer(address) then  internal_data_out <= CONV_SIGNED(X(i), p); end if;
     end loop;
   end process;  
   WITH oe SELECT data_out <= internal_data_out WHEN '1', (OTHERS => 'Z') WHEN OTHERS;

end structure;	


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
USE WORK.image_filter_parameters.ALL;
entity test_image_memory is end test_image_memory;

architecture test of test_image_memory is

component image_memory is
port (
  data_in: in signed(p-1 downto 0);
  clk, we, oe: in std_logic;
  address: in std_logic_vector(s-1 downto 0);
  data_out: out signed(p-1 downto 0)   
);
end component;

signal data_in: signed(p-1 downto 0);
signal clk: std_logic := '0';
signal we, oe: std_logic;
signal address: std_logic_vector(s-1 downto 0);
signal data_out: signed(p-1 downto 0);

begin

dut: image_memory
port map(data_in => data_in, clk => clk, we => we, oe => oe, address => address, 
data_out => data_out);
  
clk <= not(clk) after 10 ns;

process
begin
we <= '1'; oe <= '0';

data_in <= "00000000"; 
address <= "0100000000";

for j in 1 to m*n-1 loop
  wait for 100 ns;
  address <= address + 1;
  data_in <= data_in + 1;
end loop;

wait for 100 ns;
we <= '0'; oe <= '1'; 

address <= "0000000000";

for j in 0 to m*n-1 loop
  wait for 100 ns;
  address <= address + 1;
end loop;

address <= "0100000000";

for j in 0 to m*n-1 loop
  wait for 100 ns;
  address <= address + 1;
end loop;


wait;

end process;

end test;

