LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_SIGNED.ALL;
USE WORK.image_filter_parameters.ALL;
ENTITY image_processing IS END image_processing;

ARCHITECTURE data_flow OF image_processing IS

COMPONENT image_filter_DF IS
PORT (
    a, b: IN STD_LOGIC_VECTOR(s-1 DOWNTO 0);
    data_in: IN SIGNED(p-1 DOWNTO 0);
    clk, start, grant: IN STD_LOGIC;
    address: OUT STD_LOGIC_VECTOR(s-1 DOWNTO 0);
    data_out: OUT SIGNED(p-1 DOWNTO 0);
    req, done, we, oe: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT image_memory IS
PORT (
  data_in: in signed(p-1 downto 0);
  clk, we, oe: in std_logic;
  address: in std_logic_vector(s-1 downto 0);
  data_out: out signed(p-1 downto 0)   
);
END COMPONENT;

SIGNAL req, done, we, oe: STD_LOGIC;
SIGNAL address: STD_LOGIC_VECTOR(s-1 DOWNTO 0);
SIGNAL data_bus: SIGNED(p-1 DOWNTO 0);
SIGNAL a, b: STD_LOGIC_VECTOR(s-1 DOWNTO 0);
SIGNAL clk, start, grant: STD_LOGIC := '0';

BEGIN

first_component: image_filter_DF
PORT MAP(
   a => a, b => b, data_in => data_bus, clk => clk, start => start, 
   grant => grant, address => address, data_out => data_bus, req => req, 
   done => done, we => we, oe => oe);

second_component: image_memory
PORT MAP(
  data_in => data_bus, clk => clk, we => we, oe => oe,
  address => address, data_out => data_bus);

clk <= NOT(clk) AFTER 50 NS;
a <= "0000000000";
b <= "0100000000";

----CPU-----
PROCESS
BEGIN
   WAIT FOR 500 NS;
   start <= '1';
   WAIT FOR 100 NS;
   start <= '0';
   WAIT UNTIL done = '1';
   WAIT;
END PROCESS;

----BUS ARBITER----
PROCESS
BEGIN
   WAIT UNTIL req = '1';
   WAIT FOR 100 NS;
   grant <= '1';
   WAIT FOR 100 NS;
   grant <= '0';
   WAIT UNTIL done = '1';
   WAIT;
END PROCESS;

------FILTERED IMAGE READING--------
PROCESS
BEGIN
   address <= (OTHERS => 'Z');
   oe <= 'Z'; we <= 'Z';
   WAIT FOR 300000 ns;
   address <= b;
   oe <= '1'; we <= '0';
   WAIT FOR 100 NS;
   FOR i IN 1 TO m*n-1 LOOP
     address <= address + '1';
     WAIT FOR 100 NS;
   END LOOP;
   address <= (OTHERS => 'Z');
   oe <= 'Z'; we <= 'Z';
   WAIT;
END PROCESS;


END data_flow;

