LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY logarithm IS 
   GENERIC(n, m, k, p: natural);
PORT(
   x: in std_logic_vector(0 to n);
   start, clk, reset: in std_logic;
   y: out std_logic_vector(1 to p);
   done: out std_logic
);
END logarithm;

ARCHITECTURE circuit OF logarithm IS
   SIGNAL z, trunc_square, trunc_div, z_input: std_logic_vector(m+1 downto 0);
   SIGNAL i, i_input, i_minus_1: std_logic_vector(k-1 downto 0);
   SIGNAL yy: std_logic_vector(1 to p);
   SIGNAL mult_in: std_logic_vector(m downto 0);
   SIGNAL mult_out: std_logic_vector(2*m+1 downto 0);
   
   SIGNAL sel_z: std_logic_vector(1 downto 0);
   SIGNAL load_z, sel_i, load_i, shift_y: std_logic;
   SIGNAL i_positive, z_gt2: std_logic;

   CONSTANT zeroes: std_logic_vector(m-n-1 downto 0) := (others => '0'); 
   CONSTANT binary_p: std_logic_vector (k-1 downto 0) := CONV_STD_LOGIC_VECTOR(p, k);

   TYPE state IS RANGE 0 TO 6;
   SIGNAL current_state: state;

   
BEGIN

register_z: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN z <= (others => '0');
   ELSIF clk'event and clk = '1' THEN
     IF load_z = '1' THEN z <= z_input; END IF;
   END IF;
END PROCESS;

register_i: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN i <= (others => '0');
   ELSIF clk'event and clk = '1' THEN
     IF load_i = '1' THEN i <= i_input; END IF;
   END IF;
END PROCESS;

register_y: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN yy <= (others => '0');
   ELSIF clk'event and clk = '1' THEN
     IF shift_y = '1' THEN yy <= yy(2 to p)&z(m+1); END IF;
   END IF;
END PROCESS;

WITH sel_z SELECT z_input <= '0'&x&zeroes WHEN "00",
                             trunc_square WHEN "01",
                             trunc_div WHEN others;

WITH sel_i SELECT i_input <= binary_p WHEN '0', 
                             i_minus_1 WHEN others;         

mult_in <= z(m downto 0);
mult_out <= mult_in * mult_in;
trunc_square <= mult_out(2*m+1 downto m);

trunc_div <= '0'&z(m+1 downto 1);

i_minus_1 <= i-1;

or_function: PROCESS(i)
BEGIN
  IF CONV_INTEGER(i) = 0 THEN i_positive <= '0'; ELSE i_positive <= '1'; END IF;
END PROCESS;

z_gt2 <= z(m+1);

y <= yy;

next_state: PROCESS(reset, clk)
BEGIN
  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'event and clk = '1' THEN
    CASE current_state IS
        WHEN 0 => IF start = '0' THEN current_state <= 1; END IF;
        WHEN 1 => IF start = '1' THEN current_state <= 2; END IF;
        WHEN 2 => current_state <= 3;
        WHEN 3 => current_state <= 4;
        WHEN 4 => IF z_gt2 = '1' THEN current_state <= 5; ELSE current_state <= 6; END IF;
        WHEN 5 => current_state <= 6;
--        WHEN 6 => current_state <= 7;
        WHEN 6 => IF i_positive = '1' THEN current_state <= 3; ELSE current_state <= 0; END IF;
      END CASE;
    END IF;
  END PROCESS next_state;

output_state: PROCESS(current_state)
 BEGIN
   CASE current_state IS
     WHEN 0 TO 1 => sel_z <= "00"; load_z <= '0'; sel_i <= '0'; load_i <= '0'; shift_y <= '0'; done <= '1';
     WHEN 2 => sel_z <= "00"; load_z <= '1'; sel_i <= '0'; load_i <= '1'; shift_y <= '0'; done <= '0';
     WHEN 3 => sel_z <= "01"; load_z <= '1'; sel_i <= '1'; load_i <= '1'; shift_y <= '0'; done <= '0';
     WHEN 4 => sel_z <= "00"; load_z <= '0'; sel_i <= '0'; load_i <= '0'; shift_y <= '1'; done <= '0';
     WHEN 5 => sel_z <= "10"; load_z <= '1'; sel_i <= '0'; load_i <= '0'; shift_y <= '0'; done <= '0';
     WHEN 6 => sel_z <= "00"; load_z <= '0'; sel_i <= '0'; load_i <= '0'; shift_y <= '0'; done <= '0';
   END CASE;
 END PROCESS output_state;

END circuit;



LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_logarithm IS END test_logarithm;

ARCHITECTURE test OF test_logarithm IS

   COMPONENT logarithm IS   
     GENERIC(n, m, k, p: natural);
   PORT(
      x: in std_logic_vector(0 to n);
      start, clk, reset: in std_logic;
      y: out std_logic_vector(1 to p);
      done: out std_logic
   );
   END COMPONENT;

   SIGNAL x: std_logic_vector(0 to 8);
   SIGNAL y: std_logic_vector(1 to 16);
   SIGNAL clk: std_logic := '0';
   SIGNAL start, reset, done: std_logic; 

BEGIN

  dut: logarithm
     generic map(n => 8, m => 16, k => 5, p => 16)
     port map(x => x, y => y, clk => clk, start => start, reset => reset, done => done);

clk <= not clk after 50 ns;

test_vectors: PROCESS
BEGIN
--   x <= "110010110";
   x <= "110110001";
   start <= '1';
   reset <= '1';
   wait for 100 ns;
   reset <= '0';
   wait for 100 ns;
   start <= '0';
   wait for 100 ns;
   start <= '1';
   wait for 100 ns;
   start <= '0';
   wait for 5000 ns;
   start <= '1';
   wait;
END PROCESS;

END test;
