LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY logarithm_cell_DS IS 
   GENERIC(m: natural);
PORT(
   z: in std_logic_vector(m+3 downto 0);
   next_z: out std_logic_vector(m+3 downto 0);
   t: out std_logic_vector(0 to 1)
);
END logarithm_cell_DS;

ARCHITECTURE circuit OF logarithm_cell_DS IS
   SIGNAL trunc, trunc_div_8, trunc_div_4, trunc_div_2: std_logic_vector(m+3 downto 0);
   SIGNAL mult_in: std_logic_vector(m downto 0);
   SIGNAL mult_1: std_logic_vector(2*m+1 downto 0);
   SIGNAL mult_2: std_logic_vector(4*m+3 downto 0);
BEGIN

mult_in <= z(m downto 0);
mult_1 <= mult_in * mult_in;
mult_2 <= mult_1 * mult_1;

trunc <= mult_2(4*m+3 downto 3*m);
trunc_div_2 <= '0'&trunc(m+3 downto 1);
trunc_div_4 <= "00"&trunc(m+3 downto 2);
trunc_div_8 <= "000"&trunc(m+3 downto 3);

selector: PROCESS(trunc, trunc_div_2, trunc_div_4, trunc_div_8)
BEGIN
   IF trunc(m+3) = '1' THEN next_z <= trunc_div_8; t <= "11";
   ELSIF trunc(m+2) = '1' THEN next_z <= trunc_div_4; t <= "10";
   ELSIF trunc(m+1) = '1' THEN next_z <= trunc_div_2; t <= "01";
   ELSE next_z <= trunc; t <= "00";
   END IF;
END PROCESS;

END circuit;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_logarithm_cell_DS IS END test_logarithm_cell_DS;

ARCHITECTURE test OF test_logarithm_cell_DS IS

  COMPONENT logarithm_cell_DS IS 
     GENERIC(m: natural);
  PORT(
     z: in std_logic_vector(m+3 downto 0);
     next_z: out std_logic_vector(m+3 downto 0);
     t: out std_logic_vector(0 to 1)
  );
  END COMPONENT;
 
  SIGNAL z, next_z: std_logic_vector(19 downto 0);
  SIGNAL t: std_logic_vector(0 to 1);
  
  BEGIN

  dut: logarithm_cell_DS
  GENERIC MAP(m => 16)
  PORT MAP (z => z, next_z => next_z, t => t);

 z <= "00011011000100000000";
--1B100

END test;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY logarithm_DS IS 
   GENERIC(n, m, p: natural);
PORT(
   x: in std_logic_vector(0 to n);
   y: out std_logic_vector(1 to p)
);
END logarithm_DS;

ARCHITECTURE circuit OF logarithm_DS IS
   
   COMPONENT logarithm_cell_DS IS 
   GENERIC(m: natural);
   PORT(
   z: in std_logic_vector(m+3 downto 0);
   next_z: out std_logic_vector(m+3 downto 0);
   t: out std_logic_vector(0 to 1)
   );
   END COMPONENT;

   TYPE vector_data IS ARRAY (1 TO p/2) OF std_logic_vector(m+3 downto 0);
   SIGNAL vector_z: vector_data;
   CONSTANT zeroes: std_logic_vector(m-n-1 downto 0) := (others => '0'); 

BEGIN

vector_z(1) <= "000"&x&zeroes;
iteration: FOR i IN 1 TO p/2-1 GENERATE
   iterative_cell: logarithm_cell_DS
   GENERIC MAP (m => m)
   PORT MAP(z => vector_z(i), next_z => vector_z(i+1), t => y(2*i-1 to 2*i));
   END GENERATE;
last_cell: logarithm_cell_DS
   GENERIC MAP (m => m)
   PORT MAP(z => vector_z(p/2), t => y(p-1 to p));

END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_logarithm_DS IS END test_logarithm_DS;

ARCHITECTURE test OF test_logarithm_DS IS

COMPONENT logarithm_DS IS 
   GENERIC(n, m, p: natural);
PORT(
   x: in std_logic_vector(0 to n);
   y: out std_logic_vector(1 to p)
);
END COMPONENT;

   SIGNAL x: std_logic_vector(0 to 8);
   SIGNAL y: std_logic_vector(1 to 16);
BEGIN

  dut: logarithm_DS
     generic map(n => 8, m => 16, p => 16)
     port map(x => x, y => y);

test_vectors: PROCESS
BEGIN
   x <= "100000000";
   WAIT FOR 100 NS;
   x <= "110110001";
   WAIT;
END PROCESS;

END test;
