LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY logarithm_cell IS 
   GENERIC(m: natural);
PORT(
   z: in std_logic_vector(m+1 downto 0);
   next_z: out std_logic_vector(m+1 downto 0);
   t: out std_logic
);
END logarithm_cell;

ARCHITECTURE circuit OF logarithm_cell IS
   SIGNAL trunc_square, trunc_div: std_logic_vector(m+1 downto 0);
   SIGNAL mult_in: std_logic_vector(m downto 0);
   SIGNAL mult_out: std_logic_vector(2*m+1 downto 0);
BEGIN

WITH trunc_square(m+1) SELECT next_z <= trunc_square WHEN '0', 
                              trunc_div WHEN others;         

mult_in <= z(m downto 0);
mult_out <= mult_in * mult_in;
trunc_square <= mult_out(2*m+1 downto m);
t <= trunc_square(m+1);
trunc_div <= '0'&trunc_square(m+1 downto 1);

END circuit;



LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY logarithm_unfolded IS 
   GENERIC(n, m, p: natural);
PORT(
   x: in std_logic_vector(0 to n);
   y: out std_logic_vector(1 to p)
);
END logarithm_unfolded;

ARCHITECTURE circuit OF logarithm_unfolded IS
   
   COMPONENT logarithm_cell IS 
   GENERIC(m: natural);
   PORT(
   z: in std_logic_vector(m+1 downto 0);
   next_z: out std_logic_vector(m+1 downto 0);
   t: out std_logic
   );
   END COMPONENT;

   TYPE vector_data IS ARRAY (1 TO p) OF std_logic_vector(m+1 downto 0);
   SIGNAL vector_z: vector_data;
   CONSTANT zeroes: std_logic_vector(m-n-1 downto 0) := (others => '0'); 

BEGIN

vector_z(1) <= '0'&x&zeroes;
iteration: FOR i IN 1 TO p-1 GENERATE
   iterative_cell: logarithm_cell
   GENERIC MAP (m => m)
   PORT MAP(z => vector_z(i), next_z => vector_z(i+1), t => y(i));
   END GENERATE;
last_cell: logarithm_cell
   GENERIC MAP (m => m)
   PORT MAP(z => vector_z(p), t => y(p));

END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
ENTITY test_logarithm_unfolded IS END test_logarithm_unfolded;

ARCHITECTURE test OF test_logarithm_unfolded IS

COMPONENT logarithm_unfolded IS 
   GENERIC(n, m, p: natural);
PORT(
   x: in std_logic_vector(0 to n);
   y: out std_logic_vector(1 to p)
);
END COMPONENT;

   SIGNAL x: std_logic_vector(0 to 8);
   SIGNAL y: std_logic_vector(1 to 16);
BEGIN

  dut: logarithm_unfolded
     generic map(n => 8, m => 16, p => 16)
     port map(x => x, y => y);

test_vectors: PROCESS
BEGIN
   x <= "110110001";
   wait;
END PROCESS;

END test;
