library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_signed.all;
package pid_parameters is
  constant m: integer := 32;
  constant t: integer := 16;
  constant k1: signed(t-1 downto 0) := x"35b7";
  constant k2: signed(t-1 downto 0) := x"a4e9";
  constant k3: signed(t-1 downto 0) := x"2561";
end pid_parameters;

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.pid_parameters.all;
entity mac is
port (
  x: in signed(m-1 downto 0);
  y, k: in signed(t-1 downto 0);
  oper: in std_logic;
  z: out signed(m-1 downto 0)
);
end mac;

architecture behavior of mac is
  signal product: signed(m-1 downto 0); 
begin
      product <= k * y; 
      with oper select z <= x + product WHEN '0', x - product WHEN OTHERS;
end behavior;


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;
use work.pid_parameters.all;

entity test_mac is end test_mac;

architecture test of test_mac is

component mac is
port (
  x: in signed(m-1 downto 0);
  y, k: in signed(t-1 downto 0);
  oper: in std_logic;
  z: out signed(m-1 downto 0)
);
end component;

signal x, z: signed(m-1 downto 0);
signal y, k: signed(t-1 downto 0);
signal oper: std_logic;

begin

dut: mac
port map( x => x, y => y, k => k, z => z, oper => oper);

test_vectors: process
begin

oper <= '0';
x <= x"00000000";
k <= k1;
y <= x"8000";
wait for 100 ns;
x <= x"f0000000";
k <= k2;
y <= x"0001";
wait for 100 ns;
x <= x"0000"&k1;
k <= k2;
y <= x"0001";
wait for 100 ns;
x <= x"0000"&k2;
k <= k3;
y <= x"0001";
wait for 100 ns;
oper <= '1';
x <= x"00000000";
k <= k1;
y <= x"8000";
wait for 100 ns;
x <= x"f0000000";
k <= k2;
y <= x"0001";
wait for 100 ns;
x <= x"0000"&k1;
k <= k2;
y <= x"0001";
wait for 100 ns;
x <= x"0000"&k2;
k <= k3;
y <= x"0001";

wait;

end process;

end test;


