----------------------------------------------------------------------------
-- mod m product; 
-- m = 91 
--
----------------------------------------------------------------------------
--LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
--USE IEEE.STD_LOGIC_ARITH.ALL;
--USE IEEE.STD_LOGIC_UNSIGNED.ALL;
--PACKAGE mod_91 IS
--  CONSTANT m: STD_LOGIC_VECTOR(7 DOWNTO 0):= x"5b";
----minus_m = 256 - 91 = 165
--  CONSTANT minus_m: STD_LOGIC_VECTOR(7 DOWNTO 0):= x"a5";
--  CONSTANT zero: STD_LOGIC_VECTOR(7 DOWNTO 0):= x"00";
--  CONSTANT one: STD_LOGIC_VECTOR(7 DOWNTO 0):= x"01";
--END mod_91;
--
LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_91.ALL;
ENTITY mod_91_multiplier IS
PORT(
  x, y: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
  clk, reset, start: STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
  done: OUT STD_LOGIC
);
END mod_91_multiplier;

ARCHITECTURE circuit OF mod_91_multiplier IS

COMPONENT mod_91_adder IS
PORT(
x, y: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
z: OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
);
END COMPONENT;

SIGNAL adder_x, adder_z, a, b, reg_in_a, reg_in_b, internal_y: STD_LOGIC_VECTOR(7 DOWNTO 0);

SIGNAL sel, load, en_a, en_b, shift: STD_LOGIC;

SIGNAL count: NATURAL;

TYPE state IS RANGE 0 TO 7;
SIGNAL current_state: state;

BEGIN

WITH sel SELECT adder_x <= a WHEN '0', b WHEN OTHERS;

main_component: mod_91_adder
  PORT MAP( x => adder_x, y => b, z => adder_z);

WITH load SELECT reg_in_a <= adder_z WHEN '0', zero WHEN OTHERS; 
WITH load SELECT reg_in_b <= adder_z WHEN '0', x WHEN OTHERS;

registers: PROCESS(clk)
BEGIN
  IF clk'EVENT AND clk = '1' THEN
    IF en_a = '1' THEN a <= reg_in_a; END IF;
    IF en_b = '1' THEN b <= reg_in_b; END IF;
  END IF;
END PROCESS;

z <= a;

counter: PROCESS(clk)
BEGIN
  IF clk'EVENT AND clk = '1' THEN
    IF load = '1' THEN count <= 7; 
    ELSIF shift = '1' THEN count <= count - 1; END IF;
  END IF;
END PROCESS;

shift_register: PROCESS(clk)
BEGIN
  IF clk'EVENT AND clk = '1' THEN
    IF load = '1' THEN internal_y <= y; 
    ELSIF shift = '1' THEN internal_y <= '0'&internal_y(7 DOWNTO 1); END IF;
  END IF;
END PROCESS;

--control_unit
next_state: PROCESS(clk, reset, internal_y, count, start)
BEGIN
   IF reset = '1' THEN current_state <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state IS
         WHEN 0 => IF start = '0' THEN current_state <= 1; END IF;
         WHEN 1 => IF start = '1' THEN current_state <= 2; END IF;
         WHEN 2 => current_state <= 3;
         WHEN 3 => IF internal_y(0) = '1' THEN current_state <= 4; ELSE current_state <= 5; END IF;
         WHEN 4 => current_state <= 5;
         WHEN 5 => current_state <= 6;
         WHEN 6 => IF count = 0 THEN current_state <= 0; ELSE current_state <= 7; END IF;
         WHEN 7 => current_state <= 3;

      END CASE;
   END IF;
END PROCESS;

output_function: PROCESS(current_state)
BEGIN
      CASE current_state IS
         WHEN 0 TO 1 => sel <= '0'; load <= '0'; en_a <= '0'; en_b <= '0'; shift <= '0'; done <= '1';
         WHEN 2 => sel <= '0'; load <= '1'; en_a <= '1'; en_b <= '1'; shift <= '0'; done <= '0';
         WHEN 3 => sel <= '0'; load <= '0'; en_a <= '0'; en_b <= '0'; shift <= '0'; done <= '0';
         WHEN 4 => sel <= '0'; load <= '0'; en_a <= '1'; en_b <= '0'; shift <= '0'; done <= '0';
         WHEN 5 => sel <= '1'; load <= '0'; en_a <= '0'; en_b <= '1'; shift <= '0'; done <= '0';
         WHEN 6 => sel <= '0'; load <= '0'; en_a <= '0'; en_b <= '0'; shift <= '0'; done <= '0';
         WHEN 7 => sel <= '0'; load <= '0'; en_a <= '0'; en_b <= '0'; shift <= '1'; done <= '0';
      END CASE;
END PROCESS;

END circuit;  


LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work. mod_91.ALL;
ENTITY test_mod_91_multiplier IS END test_mod_91_multiplier;

ARCHITECTURE test OF test_mod_91_multiplier IS

COMPONENT mod_91_multiplier IS
PORT(
  x, y: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
  clk, reset, start: STD_LOGIC;
  z: OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
  done: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL x, y, z: STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL clk: STD_LOGIC := '0';
SIGNAL start, done, reset: STD_LOGIC;

BEGIN

dut: mod_91_multiplier
  PORT MAP(x => x, y => y, z => z, clk => clk, start => start, done => done, reset => reset);

clk <= NOT(clk) AFTER 50 NS;

test_vectors: PROCESS
BEGIN
reset <= '1'; start <= '0'; 
x <= x"43"; y <= x"25";
WAIT FOR 100 NS;
reset <= '0';
WAIT FOR 100 NS;
start <= '1';
WAIT FOR 100 NS;
start <= '0';
WAIT FOR 4000 NS;
y <= x"1f";
start <= '1';
WAIT FOR 100 NS;
start <= '0';
WAIT FOR 4000 NS;
x <= x"1f";
start <= '1';
WAIT FOR 100 NS;
start <= '0';
WAIT FOR 4000 NS;
WAIT;
END PROCESS;

--x <= x"43", x"1f" AFTER 200 NS;
--y <= x"25", x"1f" AFTER 100 NS;

END test;