library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity one_port_register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write, read: in std_logic;
  write_address, read_address: in std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(m-1 downto 0)  
);
end one_port_register_file;

architecture structure of one_port_register_file is

type memory is array (0 to n-1) of std_logic_vector(m-1 downto 0);
signal X: memory;
signal EN: std_logic_vector(0 to n-1);
signal reg_out: std_logic_vector(m-1 downto 0); 

begin

  decoder: process(write_address, write)
  begin
    for i in 0 to n-1 loop
      if i = conv_integer(write_address) then EN(i) <= write;
      else EN(i) <= '0';
      end if;
    end loop;
  end process;
  
   bank_registers: process(clk)
   begin
     if clk'event and clk = '1' then
       for i in 0 to n-1 loop
         if EN(i) = '1' then X(i) <= data_in; end if;
       end loop;
     end if;
    end process;

  multiplexer: process(read_address, X)
  begin
     for i in 0 to n-1 loop
        if i = conv_integer(read_address) then  reg_out <= X(i); end if;
     end loop;
   end process;  

  output_register: process(clk)
  begin
     if clk'event and clk = '1' then
        if read = '1' then data_out <= reg_out; end if;
     end if;
  end process;

end structure;	



