--LIBRARY IEEE;
--USE IEEE.STD_LOGIC_1164.ALL;
--USE IEEE.STD_LOGIC_ARITH.ALL;
--USE IEEE.STD_LOGIC_SIGNED.ALL;
--PACKAGE image_filter_parameters IS
--  CONSTANT m: INTEGER := 9; -- m rows
--  CONSTANT n: INTEGER := 16; -- n columns
--  CONSTANT p: INTEGER := 8; -- 8 bits per pixel
--  CONSTANT s: INTEGER := 10; -- address bits
--  TYPE memory IS ARRAY (0 TO m*n - 1) OF INTEGER;
--  CONSTANT image: memory := (
--  0, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
--  0,-1,-2,-3,-4,-5,-6,-7,-8,-7,-6,-5,-4,-3,-2,-1,
--  0, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
--  0,-1,-2,-3,-4,-5,-6,-7,-8,-7,-6,-5,-4,-3,-2,-1,
--  0, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
--  0,-1,-2,-3,-4,-5,-6,-7,-8,-7,-6,-5,-4,-3,-2,-1,
--  0, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1,
--  0,-1,-2,-3,-4,-5,-6,-7,-8,-7,-6,-5,-4,-3,-2,-1,
--  0, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1
--  );
--END image_filter_parameters;
--
library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
USE WORK.image_filter_parameters.ALL;
entity picture_memory is
port (
  data_in: in std_logic_vector(p-1 downto 0);
  clk, we, oe: in std_logic;
  address: in std_logic_vector(s-1 downto 0);
  data_out: out std_logic_vector(p-1 downto 0)  
);
end picture_memory;

architecture structure of picture_memory is

SIGNAL X: memory := image;
signal EN: std_logic_vector(0 to m*n-1);
SIGNAL internal_data_out: std_logic_vector(p-1 downto 0);  

begin

  decoder: process(address, we)
  begin
    for i in 0 to m*n-1 loop
      if i = conv_integer(address) then EN(i) <= we;
      else EN(i) <= '0';
      end if;
    end loop;
  end process;
  
   bank_registers: process(clk)
   begin
     if clk'event and clk = '1' then
       for i in 0 to m*n-1 loop
         if EN(i) = '1' then X(i) <= CONV_INTEGER(data_in); end if;
       end loop;
     end if;
    end process;

  multiplexer: process(address, X)
  begin
     for i in 0 to m*n-1 loop
        if i = conv_integer(address) AND oe = '1' then  internal_data_out <= CONV_STD_LOGIC_VECTOR(X(i), p); end if;
     end loop;
   end process;  
   WITH oe SELECT data_out <= internal_data_out WHEN '1', (OTHERS => 'Z') WHEN OTHERS;

end structure;	


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
USE WORK.image_filter_parameters.ALL;
entity test_picture_memory is end test_picture_memory;

architecture test of test_picture_memory is

component picture_memory is
port (
  data_in: in std_logic_vector(p-1 downto 0);
  clk, we, oe: in std_logic;
  address: in std_logic_vector(s-1 downto 0);
  data_out: out std_logic_vector(p-1 downto 0)   
);
end component;

signal data_in: std_logic_vector(p-1 downto 0);
signal clk: std_logic := '0';
signal we, oe: std_logic;
signal address: std_logic_vector(s-1 downto 0);
signal data_out: std_logic_vector(p-1 downto 0);

begin

dut: picture_memory
port map(data_in => data_in, clk => clk, we => we, oe => oe, address => address, 
data_out => data_out);
  
clk <= not(clk) after 10 ns;

process
begin
we <= '1'; oe <= '0';

data_in <= "00000000"; 
address <= "0000000000";

for j in 1 to m*n-1 loop
  wait for 100 ns;
  address <= address + 1;
  data_in <= data_in + 1;
end loop;

wait for 100 ns;
we <= '0'; oe <= '1'; 

address <= "0000000000";

for j in 0 to m*n-1 loop
  wait for 100 ns;
  address <= address + 1;
end loop;

wait;

end process;

end test;

