LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
USE work.main_parameters.ALL;
USE IEEE.MATH_REAL.ALL; -- for UNIFORM, TRUNC
ENTITY receiver IS
  GENERIC(key: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset, ready: IN STD_LOGIC;
    data_in: IN STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    decoded: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END receiver;

ARCHITECTURE circuit OF receiver IS

COMPONENT mod_m_exponentiation IS 
  PORT(
    x, y: IN STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    clk, reset, start: STD_LOGIC;
    z: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    done: OUT STD_LOGIC
  );
END COMPONENT;

COMPONENT fifo IS
PORT (
  data_in: in std_logic_vector(k-1 downto 0);
  clk, reset, write, read: in std_logic;
  empty, full: out std_logic;
--  s: inout std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(k-1 downto 0)
);  
END COMPONENT;  


SIGNAL fifo_out, z: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL start_exp, exp_done, full, empty, write_fifo, read_fifo: STD_LOGIC;

TYPE state1 IS RANGE 0 TO 3;
SIGNAL current_state1: state1;

TYPE state2 IS RANGE 0 TO 3;
SIGNAL current_state2: state2;


BEGIN

first_component: fifo
  PORT MAP (data_in => data_in, clk => clk, reset => reset, write => write_fifo, 
    read => read_fifo, empty => empty, full => full, data_out => fifo_out);

second_component: mod_m_exponentiation
  PORT MAP(x => key, y => fifo_out, clk => clk, reset => reset, 
    start => start_exp, z => z, done => exp_done);

output_register: PROCESS (exp_done)
BEGIN
  IF exp_done'event AND exp_done = '1' THEN decoded <= z; END IF;
END PROCESS;

-- FIFO write
next_state1: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state1 <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state1 IS
         WHEN 0 => IF ready = '0' THEN current_state1 <= 1; END IF;
         WHEN 1 => IF ready = '1' AND full = '0' THEN current_state1 <= 2; 
                   ELSIF ready = '1' AND full = '1' THEN current_state1 <= 3; END IF;
         WHEN 2 => current_state1 <= 0;
         WHEN 3 => current_state1 <= 0;
      END CASE;
   END IF;
END PROCESS;

output_function1: PROCESS(current_state1)
BEGIN
      CASE current_state1 IS
         WHEN 0 TO 1 => write_fifo <= '0';
         WHEN 2 => write_fifo <= '1'; 
         WHEN 3 => write_fifo <= '0'; 
      END CASE;
END PROCESS;

-- FIFO read         
next_state2: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state2 <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state2 IS
         WHEN 0 => IF empty = '0' THEN current_state2 <= 1; END IF;
         WHEN 1 => current_state2 <= 2;
         WHEN 2 => current_state2 <= 3;
         WHEN 3 => IF exp_done = '1' THEN current_state2 <= 0; END IF;
      END CASE;
   END IF;
END PROCESS;

output_function2: PROCESS(current_state2)
BEGIN
      CASE current_state2 IS
         WHEN 0 => read_fifo <= '0'; start_exp <= '0';
         WHEN 1 => read_fifo <= '1'; start_exp <= '0';
         WHEN 2 => read_fifo <= '0'; start_exp <= '1';
         WHEN 3 => read_fifo <= '0'; start_exp <= '0';
      END CASE;
END PROCESS;

END circuit;


