library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
package main_parameters is
  constant m: natural := 8; -- m bits per word
  constant n: natural := 16; -- n words
  constant p: natural := 4; -- p address bits
  constant zero: std_logic_vector(m-1 downto 0) := conv_std_logic_vector(0, m);
end main_parameters;

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write: in std_logic;
  address: in std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(m-1 downto 0)  
);
end register_file;

architecture structure of register_file is

type memory is array (0 to n-1) of std_logic_vector(m-1 downto 0);
signal X: memory;
signal EN: std_logic_vector(0 to n-1);

begin

  decoder: process(address, write)
  begin
    for i in 0 to n-1 loop
      if i = conv_integer(address) then EN(i) <= write;
      else EN(i) <= '0';
      end if;
    end loop;
  end process;
  
   bank_registers: process(clk)
   begin
     if clk'event and clk = '1' then
       for i in 0 to n-1 loop
         if EN(i) = '1' then X(i) <= data_in; end if;
       end loop;
     end if;
    end process;

  multiplexer: process(address, X)
  begin
     for i in 0 to n-1 loop
        if i = conv_integer(address) then  data_out <= X(i); end if;
     end loop;
   end process;  
	
end structure;	


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity test_register_file is end test_register_file;

architecture test of test_register_file is

component register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write: in std_logic;
  address: in std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(m-1 downto 0)  
);
end component;

signal data_in: std_logic_vector(m-1 downto 0);
signal clk: std_logic := '0';
signal write: std_logic;
signal address: std_logic_vector(p-1 downto 0);
signal data_out: std_logic_vector(m-1 downto 0);

begin

dut: register_file
port map(data_in => data_in, clk => clk, write => write, address => address, 
data_out => data_out);
  
clk <= not(clk) after 10 ns;

process
begin
data_in <= "00000000"; 
write <= '1';
address <= "0000";

for j in 0 to n-1 loop
  wait for 100 ns;
  address <= address + 1;
  data_in <= data_in + 1;
end loop;

write <= '0';
for j in 0 to n-1 loop
  wait for 100 ns;
  address <= address + 1;
end loop;

write <= '1';
for p in 0 to n-1 loop
  wait for 100 ns;
  address <= address + 1;
  data_in <= data_in - 1;
end loop;

wait;

end process;

end test;

