LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
USE IEEE.MATH_REAL.ALL; -- for UNIFORM, TRUNC
ENTITY rsa IS
  GENERIC(public, private: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset: IN STD_LOGIC;
    message, encoded, decoded: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END rsa;

ARCHITECTURE circuit OF rsa IS

COMPONENT signal_generator IS 
  PORT(
   clk, reset, done: IN STD_LOGIC;
    random: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    start: OUT STD_LOGIC
  );
END COMPONENT;

COMPONENT mod_m_exponentiation IS 
  PORT(
    x, y: IN STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    clk, reset, start: STD_LOGIC;
    z: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    done: OUT STD_LOGIC
  );
END COMPONENT;

COMPONENT fifo IS
PORT (
  data_in: in std_logic_vector(k-1 downto 0);
  clk, reset, write, read: in std_logic;
  empty, full: out std_logic;
  s: inout std_logic_vector(p-1 downto 0);
  data_out: out std_logic_vector(k-1 downto 0)
);  
END COMPONENT;  


SIGNAL random, fifo_in, fifo_out, z: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL start_exp1, exp1_done, start_exp2, exp2_done, 
       full, empty, write_fifo, read_fifo: STD_LOGIC;

TYPE state1 IS RANGE 0 TO 3;
SIGNAL current_state1: state1;

SIGNAL count: natural;

TYPE state2 IS RANGE 0 TO 3;
SIGNAL current_state2: state2;


BEGIN

first_component: signal_generator
  PORT MAP(clk => clk, reset => reset, done => exp1_done,
    random => random, start => start_exp1);

second_component: mod_m_exponentiation 
  PORT MAP(x => public, y => random, clk => clk, reset => reset, 
    start => start_exp1, z => fifo_in, done => exp1_done);

third_component: fifo
  PORT MAP (data_in => fifo_in, clk => clk, reset => reset, write => write_fifo, 
    read => read_fifo, empty => empty, full => full, data_out => fifo_out);

fourth_component: mod_m_exponentiation
  PORT MAP(x => private, y => fifo_out, clk => clk, reset => reset, 
    start => start_exp2, z => z, done => exp2_done);

output_register: PROCESS (exp2_done)
BEGIN
  IF exp2_done'event AND exp2_done = '1' THEN decoded <= z; END IF;
END PROCESS;

message <= random;
encoded <= fifo_out;

-- FIFO write
next_state1: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state1 <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state1 IS
         WHEN 0 => IF exp1_done = '0' THEN current_state1 <= 1; END IF;
         WHEN 1 => IF exp1_done = '1' AND full = '0' THEN current_state1 <= 2; 
                   ELSIF exp1_done = '1' AND full = '1' THEN current_state1 <= 3; END IF;
         WHEN 2 => current_state1 <= 0;
         WHEN 3 => current_state1 <= 0;
      END CASE;
   END IF;
END PROCESS;

output_function1: PROCESS(current_state1)
BEGIN
      CASE current_state1 IS
         WHEN 0 TO 1 => write_fifo <= '0';
         WHEN 2 => write_fifo <= '1'; 
         WHEN 3 => write_fifo <= '0'; 
      END CASE;
END PROCESS;

-- FIFO read         
next_state2: PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN current_state2 <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state2 IS
         WHEN 0 => IF empty = '0' THEN current_state2 <= 1; END IF;
         WHEN 1 => current_state2 <= 2;
         WHEN 2 => current_state2 <= 3;
         WHEN 3 => IF exp2_done = '1' THEN current_state2 <= 0; END IF;
      END CASE;
   END IF;
END PROCESS;

output_function2: PROCESS(current_state2)
BEGIN
      CASE current_state2 IS
         WHEN 0 => read_fifo <= '0'; start_exp2 <= '0';
         WHEN 1 => read_fifo <= '1'; start_exp2 <= '0';
         WHEN 2 => read_fifo <= '0'; start_exp2 <= '1';
         WHEN 3 => read_fifo <= '0'; start_exp2 <= '0';
      END CASE;
END PROCESS;

END circuit;



LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
ENTITY test_rsa IS END test_rsa;

ARCHITECTURE test OF test_rsa IS

COMPONENT rsa IS
  GENERIC(public, private: STD_LOGIC_VECTOR(k-1 DOWNTO 0));
  PORT(
    clk, reset: IN STD_LOGIC;
    message, encoded, decoded: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0)
);
END COMPONENT;

SIGNAL clk: STD_LOGIC := '0';
SIGNAL reset: STD_LOGIC;
SIGNAL message, encoded, decoded: STD_LOGIC_VECTOR(k-1 DOWNTO 0);


BEGIN

dut: rsa
  GENERIC MAP(public => x"05", private => x"1d")
  PORT MAP(
    clk => clk, reset => reset, message => message, encoded => encoded, decoded => decoded); 

clk <= NOT(clk) AFTER 50 NS;

test_vectors: PROCESS
BEGIN
reset <= '1';
WAIT FOR 100 NS;
reset <= '0';
WAIT FOR 100 NS;

WAIT FOR 1200000 NS;

WAIT;

END PROCESS;


END test;
