LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY scalar_product IS
PORT (
    xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    clk, reset, start: IN STD_LOGIC;
    xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    done: OUT STD_LOGIC );
END scalar_product;

ARCHITECTURE circuit OF scalar_product IS

COMPONENT scalar_product_data_path IS
PORT(
  xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset, start_mult, load, shift, en_xA, en_xB, en_zA, en_zB, en_R: IN STD_LOGIC;
  sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  sel_R: IN STD_LOGIC;
  xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  count: INOUT NATURAL RANGE 0 TO M-1; 
  msb_k, mult_done: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL start_mult, mult_done, load, en_xA, en_xB, en_zA, en_zB, en_R, shift, sel_R, msb_k: STD_LOGIC;
SIGNAL sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL count: natural range 0 to M-1;

SIGNAL current_state: NATURAL RANGE 0 TO 42;

BEGIN

main_component: scalar_product_data_path PORT MAP(
  xP => xP, k => k, clk => clk, reset => reset, start_mult => start_mult, 
  load => load, shift => shift, en_xA => en_xA, en_xB => en_xB, en_zA => en_zA , en_zB => en_zB, en_R => en_R, 
  sel_p1 => sel_p1, sel_p2 => sel_p2, sel_a1 => sel_a1, sel_a2 => sel_a2,
  sel_sq => sel_sq, sel_xA => sel_xA, sel_xB => sel_xB, sel_zA => sel_zA, sel_zB => sel_zB, sel_R => sel_R,
  xA => xA, zA => zA, xB => xB, zB => zB, count => count, msb_k => msb_k, mult_done => mult_done
);  

control_unit: PROCESS(clk, reset, current_state, count)
BEGIN
  CASE current_state IS
  WHEN 0 to 1 => 
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '1';
  WHEN 2 =>
    start_mult <= '0'; load <= '1'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 3 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 4 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "01"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 5 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; done <= '0';
  WHEN 6 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; done <= '0';
  WHEN 7 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 8 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 9 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 10 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 11 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 12 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "10"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 13 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 14 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 15 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 16 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "01"; sel_p2 <= "10"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "10"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; done <= '0';
  WHEN 17 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 18 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 19 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 20 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 21 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 22 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "10"; 
    sel_xB <= "01"; sel_zA <= "10"; sel_zB <= "10"; sel_R <= '0'; done <= '0';

  WHEN 23 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "01"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 24 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 25 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 26 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 27 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 28 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 29 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 30 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 31 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "10"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 32 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; done <= '0';
  WHEN 33 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 34 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 35 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "10"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "11"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; done <= '0';
  WHEN 36 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 37 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 38 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 39 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 40 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 41 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "01"; 
    sel_xB <= "10"; sel_zA <= "11"; sel_zB <= "11"; sel_R <= '0'; done <= '0';
  WHEN 42 =>
    start_mult <= '0'; load <= '0'; 
    if count < m-1 then shift <= '1'; else shift <= '0'; end if;
    en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
END CASE;

  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'EVENT AND clk = '1' THEN
    CASE current_state IS
    WHEN 0 => IF start = '0' THEN current_state <= current_state + 1; END IF;
    WHEN 1 => IF start = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 2 => current_state <= current_state + 1;
    WHEN 3 => IF msb_k = '0' THEN current_state <= 4; ELSE current_state <= 23; END IF;
    WHEN 4 to 6 => current_state <= current_state + 1;
    WHEN 7 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 8 to 9 => current_state <= current_state + 1;
    WHEN 10 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 11 to 13 => current_state <= current_state + 1;
    WHEN 14 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 15 to 16 => current_state <= current_state + 1;
    WHEN 17 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 18 to 19 => current_state <= current_state + 1;
    WHEN 20 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 21 => current_state <= current_state + 1;
    WHEN 22 => current_state <= 42;
 
    WHEN 23 to 25 => current_state <= current_state + 1;
    WHEN 26 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 27 to 28 => current_state <= current_state + 1;
    WHEN 29 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 30 to 32 => current_state <= current_state + 1;
    WHEN 33 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 34 to 35 => current_state <= current_state + 1;
    WHEN 36 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 37 to 38 => current_state <= current_state + 1;
    WHEN 39 => IF mult_done = '1' THEN current_state <= current_state + 1; END IF;
    WHEN 40 => current_state <= current_state + 1;
    WHEN 41 => current_state <= current_state + 1; 

    WHEN 42 => IF count < m-1 THEN current_state <= 3; ELSE current_state <= 0; END IF;

    END CASE;
  END IF;  
END PROCESS;    
END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY test_scalar_product IS END test_scalar_product;

ARCHITECTURE test OF test_scalar_product IS

  COMPONENT scalar_product IS
  PORT (
      xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      clk, reset, start: IN STD_LOGIC;
      xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      done: OUT STD_LOGIC );
  END COMPONENT;

  SIGNAL xP, k: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL reset, start: STD_LOGIC;
  SIGNAL clk: STD_LOGIC := '0';
  SIGNAL xA, zA, xB, zB: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL done: STD_LOGIC ;

BEGIN

dut: scalar_product
PORT MAP (xP => xP, k => k, clk => clk, reset => reset, start => start,
xA => xA, xB => xB, zA => zA, zB => zB, done => done);

clk <= not(clk) AFTER 50 ns;

test_vectors: PROCESS
BEGIN
xP <= "010" & x"fe13c0537bbc11acaa07d793de4e6d5e5c94eee8";
k <= "100" & x"000000000000000000020108a2e0cc0d99f8a5ee";
reset <= '1';
start <= '0';
WAIT FOR 100 ns;
reset <= '0';
WAIT FOR 100 ns;
start <= '1';
WAIT FOR 100 ns;
start <= '0';
WAIT;
END PROCESS;

END test;

