LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY scalar_product_decoder_M IS
PORT (
    xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    clk, reset, start: IN STD_LOGIC;
    xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    done: OUT STD_LOGIC );
END scalar_product_decoder_M;

ARCHITECTURE circuit OF scalar_product_decoder_M IS

COMPONENT scalar_product_data_path IS
PORT(
  xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset, start_mult, load, shift, en_xA, en_xB, en_zA, en_zB, en_R: IN STD_LOGIC;
  sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  sel_R: IN STD_LOGIC;
  xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  count: INOUT NATURAL RANGE 0 TO M-1; 
  msb_k, mult_done: OUT STD_LOGIC
);
END COMPONENT;

SIGNAL start_mult, mult_done, load, en_xA, en_xB, en_zA, en_zB, en_R, shift, sel_R, msb_k: STD_LOGIC;
SIGNAL sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL count: natural range 0 to M-1;

SIGNAL current_state: NATURAL RANGE 0 TO 45;

type instruction_set is (
nop, sw_reset, multAB1, squareB, updateR, 
multBA1, updateB, multAA, squareA, updateA, 
multRB, multPA, updateAB, multBA2, multAB2, 
multBB, multRA, multPB, updateBA, inc);

signal mnemonic: instruction_set;


BEGIN

main_component: scalar_product_data_path PORT MAP(
  xP => xP, k => k, clk => clk, reset => reset, start_mult => start_mult, 
  load => load, shift => shift, en_xA => en_xA, en_xB => en_xB, en_zA => en_zA , en_zB => en_zB, en_R => en_R, 
  sel_p1 => sel_p1, sel_p2 => sel_p2, sel_a1 => sel_a1, sel_a2 => sel_a2,
  sel_sq => sel_sq, sel_xA => sel_xA, sel_xB => sel_xB, sel_zA => sel_zA, sel_zB => sel_zB, sel_R => sel_R,
  xA => xA, zA => zA, xB => xB, zB => zB, count => count, msb_k => msb_k, mult_done => mult_done
);  

--decoder that generates control signals
decoder: PROCESS(mnemonic)
BEGIN
  CASE mnemonic IS
  WHEN nop => 
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; 
    en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN sw_reset =>
    start_mult <= '0'; load <= '1'; shift <= '0'; en_xA <= '0'; 
    en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multAB1 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "01"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN squareB =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; 
  WHEN updateR =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multBA1 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateB =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multAA =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "10"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN squareA =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateA =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multRB =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "01"; sel_p2 <= "10"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "10"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; 
  WHEN multPA =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateAB =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB 
    <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "10"; 
    sel_xB <= "01"; sel_zA <= "10"; sel_zB <= "10"; sel_R <= '0'; 
  WHEN multBA2 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "01"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multAB2 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multBB =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "10"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multRA =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "10"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "11"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; 
  WHEN multPB =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateBA =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB 
    <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "01"; 
    sel_xB <= "10"; sel_zA <= "11"; sel_zB <= "11"; sel_R <= '0'; 
  WHEN inc =>
    start_mult <= '0'; load <= '0'; shift <= '1'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
END CASE;
END PROCESS decoder;

control_unit: PROCESS(clk, reset, current_state, count)
BEGIN
  --generation of encoded command 
  CASE current_state IS
  WHEN 0 to 1 => mnemonic <= nop; done <= '1';
  WHEN 2 => mnemonic <= sw_reset; done <= '0';
  WHEN 3 => mnemonic <= nop; done <= '0';
  WHEN 4 => mnemonic <= multAB1; done <= '0';
  WHEN 5 => mnemonic <= squareB; done <= '0';
  WHEN 6 => mnemonic <= squareB; done <= '0';
  WHEN 7 => mnemonic <= nop; done <= '0';
  WHEN 8 => mnemonic <= updateR; done <= '0';
  WHEN 9 => mnemonic <= multBA1; done <= '0';
  WHEN 10 => mnemonic <= nop; done <= '0';
  WHEN 11 => mnemonic <= updateB; done <= '0';
  WHEN 12 => mnemonic <= multAA; done <= '0';
  WHEN 13 => mnemonic <= squareA; done <= '0';
  WHEN 14 => mnemonic <= nop; done <= '0';
  WHEN 15 => mnemonic <= updateA; done <= '0';
  WHEN 16 => mnemonic <= multRB; done <= '0';
  WHEN 17 => mnemonic <= nop; done <= '0';
  WHEN 18 => mnemonic <= updateB; done <= '0';
  WHEN 19 => mnemonic <= multPA; done <= '0';
  WHEN 20 => mnemonic <= nop; done <= '0';
  WHEN 21 => mnemonic <= updateA; done <= '0';

  WHEN 22 => mnemonic <= updateAB; done <= '0';
  WHEN 23 => mnemonic <= nop; done <= '0';

  WHEN 24 => mnemonic <= multBA2; done <= '0';
  WHEN 25 => mnemonic <= squareA; done <= '0';
  WHEN 26 => mnemonic <= squareA; done <= '0';
  WHEN 27 => mnemonic <= nop; done <= '0';
  WHEN 28 => mnemonic <= updateR; done <= '0';
  WHEN 29 => mnemonic <= multAB2; done <= '0';
  WHEN 30 => mnemonic <= nop; done <= '0';
  WHEN 31 => mnemonic <= updateA; done <= '0';
  WHEN 32 => mnemonic <= multBB; done <= '0';
  WHEN 33 => mnemonic <= squareB; done <= '0';
  WHEN 34 => mnemonic <= nop; done <= '0';
  WHEN 35 => mnemonic <= updateB; done <= '0';
  WHEN 36 => mnemonic <= multRA; done <= '0';
  WHEN 37 => mnemonic <= nop; done <= '0';
  WHEN 38 => mnemonic <= updateA; done <= '0';
  WHEN 39 => mnemonic <= multPB; done <= '0';
  WHEN 40 => mnemonic <= nop; done <= '0';
  WHEN 41 => mnemonic <= updateB; done <= '0';
  WHEN 42 => mnemonic <= updateBA; done <= '0';

  WHEN 43 => mnemonic <= nop; done <= '0';
  WHEN 44 => mnemonic <= inc; done <= '0';
  WHEN 45 => mnemonic <= nop; done <= '0';

END CASE;
--end of encoded command generation

  IF reset = '1' THEN current_state <= 0;
  ELSIF clk'EVENT AND clk = '1' THEN
    CASE current_state IS
    WHEN 0 => IF start = '0' THEN current_state <= current_state + 1; 
      END IF;
    WHEN 1 => IF start = '1' THEN current_state <= current_state + 1; 
      END IF;
    WHEN 2 => current_state <= current_state + 1;
    WHEN 3 => IF msb_k = '0' THEN current_state <= 4; 
      ELSE current_state <= 24; END IF;
    WHEN 4 to 6 => current_state <= current_state + 1;
    WHEN 7 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 8 to 9 => current_state <= current_state + 1;
    WHEN 10 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 11 to 13 => current_state <= current_state + 1;
    WHEN 14 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 15 to 16 => current_state <= current_state + 1;
    WHEN 17 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 18 to 19 => current_state <= current_state + 1;
    WHEN 20 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 21 => current_state <= current_state + 1;
    WHEN 22 => current_state <= current_state + 1;
    WHEN 23 => current_state <= 43;
    WHEN 24 to 26 => current_state <= current_state + 1;
    WHEN 27 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 28 to 29 => current_state <= current_state + 1;
    WHEN 30 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 31 to 33 => current_state <= current_state + 1;
    WHEN 34 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 35 to 36 => current_state <= current_state + 1;
    WHEN 37 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 38 to 39 => current_state <= current_state + 1;
    WHEN 40 => IF mult_done = '1' THEN 
      current_state <= current_state + 1; END IF;
    WHEN 41 => current_state <= current_state + 1;
    WHEN 42 => current_state <= current_state + 1; 

    WHEN 43 => IF count = m-1 THEN current_state <= 0; 
       ELSE current_state <= current_state +1; END IF;
    WHEN 44 => current_state <= current_state + 1;
    WHEN 45 => current_state <= 3;

    END CASE;
  END IF;  
END PROCESS;    





END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY test_scalar_product_decoder_M IS END test_scalar_product_decoder_M;

ARCHITECTURE test OF test_scalar_product_decoder_M IS

  COMPONENT scalar_product_decoder_M IS
  PORT (
      xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      clk, reset, start: IN STD_LOGIC;
      xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      done: OUT STD_LOGIC );
  END COMPONENT;

  SIGNAL xP, k: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL reset, start: STD_LOGIC;
  SIGNAL clk: STD_LOGIC := '0';
  SIGNAL xA, zA, xB, zB: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL done: STD_LOGIC ;

BEGIN

dut: scalar_product_decoder_M
PORT MAP (xP => xP, k => k, clk => clk, reset => reset, start => start,
xA => xA, xB => xB, zA => zA, zB => zB, done => done);

clk <= not(clk) AFTER 50 ns;

test_vectors: PROCESS
BEGIN
xP <= "010" & x"fe13c0537bbc11acaa07d793de4e6d5e5c94eee8";
k <= "100" & x"000000000000000000020108a2e0cc0d99f8a5ee";
reset <= '1';
start <= '0';
WAIT FOR 100 ns;
reset <= '0';
WAIT FOR 100 ns;
start <= '1';
WAIT FOR 100 ns;
start <= '0';
WAIT;
END PROCESS;

END test;

