LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY scalar_product_decoder_micro IS
PORT (
    xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    clk, reset, start: IN STD_LOGIC;
    xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    done: OUT STD_LOGIC );
END scalar_product_decoder_micro;

ARCHITECTURE circuit OF scalar_product_decoder_micro IS

CONSTANT nop: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00000";
CONSTANT sw_reset: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00001";
CONSTANT multAB1: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00010";
CONSTANT squareB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00011";
CONSTANT updateR: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00100";
CONSTANT multBA1: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00101";
CONSTANT updateB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00110";
CONSTANT multAA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00111";
CONSTANT squareA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01000";
CONSTANT updateA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01001";
CONSTANT multRB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01010";
CONSTANT multPA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01011";
CONSTANT updateAB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01100";
CONSTANT multBA2: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01101";
CONSTANT multAB2: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01110";
CONSTANT multBB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01111";
CONSTANT multRA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10000";
CONSTANT multPB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10001";
CONSTANT updateBA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10010";
CONSTANT inc: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10011";

COMPONENT scalar_product_data_path IS
PORT(
  xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset, start_mult, load, shift, en_xA, en_xB, en_zA, en_zB, en_R: IN STD_LOGIC;
  sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  sel_R: IN STD_LOGIC;
  xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  count: INOUT NATURAL RANGE 0 TO M-1; 
  msb_k, mult_done: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT sequencer IS
PORT (
    u_type: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    jump_num: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    clk, reset, start, msb_k, mult_done: IN STD_LOGIC;
    count: IN NATURAL RANGE 0 TO M-1;
    num: INOUT STD_LOGIC_VECTOR(5 DOWNTO 0));
END COMPONENT;

COMPONENT uprogram_memory2 IS
PORT (
    address: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    u_type: OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    num_mnemo: OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    done: OUT STD_LOGIC);
END COMPONENT;

SIGNAL u_type: STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL cond_id: STD_LOGIC_VECTOR(2 DOWNTO 0);
SIGNAL address, num_mnemo: STD_LOGIC_VECTOR(5 DOWNTO 0);
SIGNAL mnemonic: STD_LOGIC_VECTOR(4 DOWNTO 0);
SIGNAL start_mult, mult_done, load, en_xA, en_xB, en_zA, en_zB, en_R, shift, sel_R, msb_k: STD_LOGIC;
SIGNAL sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL count: natural range 0 to M-1;

BEGIN

first_component: scalar_product_data_path PORT MAP(
  xP => xP, k => k, clk => clk, reset => reset, start_mult => start_mult, 
  load => load, shift => shift, en_xA => en_xA, en_xB => en_xB, en_zA => en_zA , en_zB => en_zB, en_R => en_R, 
  sel_p1 => sel_p1, sel_p2 => sel_p2, sel_a1 => sel_a1, sel_a2 => sel_a2,
  sel_sq => sel_sq, sel_xA => sel_xA, sel_xB => sel_xB, sel_zA => sel_zA, sel_zB => sel_zB, sel_R => sel_R,
  xA => xA, zA => zA, xB => xB, zB => zB, count => count, msb_k => msb_k, mult_done => mult_done
);  

second_component: uprogram_memory2 PORT MAP(
    address => address, u_type => u_type, cond_id => cond_id, num_mnemo => num_mnemo, done => done);

third_component: sequencer PORT MAP (
    u_type => u_type, cond_id => cond_id, jump_num => num_mnemo, clk => clk, reset => reset, 
    start => start, msb_k => msb_k, mult_done => mult_done, count => count, num => address);

WITH u_type SELECT mnemonic <= num_mnemo(4 DOWNTO 0) WHEN "00", nop WHEN OTHERS;  

--decoder that generates control signals
decoder: PROCESS(mnemonic)
BEGIN
  CASE mnemonic IS
  WHEN nop => 
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; 
    en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN sw_reset =>
    start_mult <= '0'; load <= '1'; shift <= '0'; en_xA <= '0'; 
    en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multAB1 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "01"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN squareB =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; 
  WHEN updateR =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multBA1 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateB =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multAA =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "10"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN squareA =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateA =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multRB =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "01"; sel_p2 <= "10"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "10"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; 
  WHEN multPA =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateAB =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB 
    <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "10"; 
    sel_xB <= "01"; sel_zA <= "10"; sel_zB <= "10"; sel_R <= '0'; 
  WHEN multBA2 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "01"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multAB2 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multBB =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "10"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN multRA =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "10"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "11"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; 
  WHEN multPB =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "01"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
  WHEN updateBA =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB 
    <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "01"; 
    sel_xB <= "10"; sel_zA <= "11"; sel_zB <= "11"; sel_R <= '0'; 
  WHEN OTHERS =>
    start_mult <= '0'; load <= '0'; shift <= '1'; en_xA <= '0'; en_xB 
    <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; 
    sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; 
END CASE;
END PROCESS decoder;

END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY test_scalar_product_decoder_micro IS END test_scalar_product_decoder_micro;

ARCHITECTURE test OF test_scalar_product_decoder_micro IS

  COMPONENT scalar_product_decoder_micro IS
  PORT (
      xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      clk, reset, start: IN STD_LOGIC;
      xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      done: OUT STD_LOGIC );
  END COMPONENT;

  SIGNAL xP, k: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL reset, start: STD_LOGIC;
  SIGNAL clk: STD_LOGIC := '0';
  SIGNAL xA, zA, xB, zB: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL done: STD_LOGIC ;

BEGIN

dut: scalar_product_decoder_micro
PORT MAP (xP => xP, k => k, clk => clk, reset => reset, start => start,
xA => xA, xB => xB, zA => zA, zB => zB, done => done);

clk <= not(clk) AFTER 50 ns;

test_vectors: PROCESS
BEGIN
xP <= "010" & x"fe13c0537bbc11acaa07d793de4e6d5e5c94eee8";
k <= "100" & x"000000000000000000020108a2e0cc0d99f8a5ee";
reset <= '1';
start <= '0';
WAIT FOR 100 ns;
reset <= '0';
WAIT FOR 100 ns;
start <= '1';
WAIT FOR 100 ns;
start <= '0';
WAIT;
END PROCESS;

END test;

