LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY scalar_product_micro IS
PORT (
    xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    clk, reset, start: IN STD_LOGIC;
    xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
    done: OUT STD_LOGIC );
END scalar_product_micro;

ARCHITECTURE circuit OF scalar_product_micro IS

COMPONENT scalar_product_data_path IS
PORT(
  xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  clk, reset, start_mult, load, shift, en_xA, en_xB, en_zA, en_zB, en_R: IN STD_LOGIC;
  sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
  sel_R: IN STD_LOGIC;
  xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  count: INOUT NATURAL RANGE 0 TO M-1; 
  msb_k, mult_done: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT sequencer IS
PORT (
    u_type: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    jump_num: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    clk, reset, start, msb_k, mult_done: IN STD_LOGIC;
    count: IN NATURAL RANGE 0 TO M-1;
    num: INOUT STD_LOGIC_VECTOR(5 DOWNTO 0));
END COMPONENT;

COMPONENT uprogram_memory IS
PORT (
    address: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    u_type: OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    jump_num: OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    --command
    start_mult, load, en_xA, en_xB, en_zA, en_zB, en_R, shift, sel_R, done: OUT STD_LOGIC;
    sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: 
       OUT STD_LOGIC_VECTOR(1 DOWNTO 0));
END COMPONENT;

SIGNAL u_type: STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL cond_id: STD_LOGIC_VECTOR(2 DOWNTO 0);
SIGNAL address, jump_num: STD_LOGIC_VECTOR(5 DOWNTO 0);

SIGNAL start_mult, mult_done, load, en_xA, en_xB, en_zA, en_zB, en_R, shift, sel_R, msb_k: STD_LOGIC;
SIGNAL sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL count: natural range 0 to M-1;

BEGIN

first_component: scalar_product_data_path PORT MAP(
  xP => xP, k => k, clk => clk, reset => reset, start_mult => start_mult, 
  load => load, shift => shift, en_xA => en_xA, en_xB => en_xB, en_zA => en_zA , en_zB => en_zB, en_R => en_R, 
  sel_p1 => sel_p1, sel_p2 => sel_p2, sel_a1 => sel_a1, sel_a2 => sel_a2,
  sel_sq => sel_sq, sel_xA => sel_xA, sel_xB => sel_xB, sel_zA => sel_zA, sel_zB => sel_zB, sel_R => sel_R,
  xA => xA, zA => zA, xB => xB, zB => zB, count => count, msb_k => msb_k, mult_done => mult_done
);  

--microprogrammed control unit
second_component: uprogram_memory PORT MAP(
    address => address, u_type => u_type, cond_id => cond_id, jump_num => jump_num, 
    start_mult => start_mult, load => load, en_xA => en_xA, en_xB => en_xB, en_zA => en_zA, 
    en_zB => en_zB, en_R => en_R, shift => shift, sel_R => sel_R, done => done,
    sel_p1 => sel_p1, sel_p2 => sel_p2, sel_a1 => sel_a1, sel_a2 => sel_a2, sel_sq => sel_sq, 
    sel_xA => sel_xA, sel_xB => sel_xB, sel_zA => sel_zA, sel_zB => sel_zB);

third_component: sequencer PORT MAP (
    u_type => u_type, cond_id => cond_id, jump_num => jump_num, clk => clk, reset => reset, 
    start => start, msb_k => msb_k, mult_done => mult_done, count => count, num => address);

END circuit;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY test_scalar_product_micro IS END test_scalar_product_micro;

ARCHITECTURE test OF test_scalar_product_micro IS

  COMPONENT scalar_product_micro IS
  PORT (
      xP, k: IN STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      clk, reset, start: IN STD_LOGIC;
      xA, zA, xB, zB: INOUT STD_LOGIC_VECTOR(M-1 DOWNTO 0);
      done: OUT STD_LOGIC );
  END COMPONENT;

  SIGNAL xP, k: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL reset, start: STD_LOGIC;
  SIGNAL clk: STD_LOGIC := '0';
  SIGNAL xA, zA, xB, zB: STD_LOGIC_VECTOR(M-1 DOWNTO 0);
  SIGNAL done: STD_LOGIC ;

BEGIN

dut: scalar_product_micro
PORT MAP (xP => xP, k => k, clk => clk, reset => reset, start => start,
xA => xA, xB => xB, zA => zA, zB => zB, done => done);

clk <= not(clk) AFTER 50 ns;

test_vectors: PROCESS
BEGIN
xP <= "010" & x"fe13c0537bbc11acaa07d793de4e6d5e5c94eee8";
k <= "100" & x"000000000000000000020108a2e0cc0d99f8a5ee";
reset <= '1';
start <= '0';
WAIT FOR 100 ns;
reset <= '0';
WAIT FOR 100 ns;
start <= '1';
WAIT FOR 100 ns;
start <= '0';
WAIT;
END PROCESS;

END test;

