LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY sequencer IS
PORT (
    u_type: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: IN STD_LOGIC_VECTOR(2 DOWNTO 0);
    jump_num: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    clk, reset, start, msb_k, mult_done: IN STD_LOGIC;
    count: IN NATURAL RANGE 0 TO M-1;
    num: INOUT STD_LOGIC_VECTOR(5 DOWNTO 0));
END sequencer;

ARCHITECTURE circuit OF sequencer IS

SIGNAL cond: STD_LOGIC;
SIGNAL next_num, plus_1: STD_LOGIC_VECTOR(5 DOWNTO 0);
SIGNAL mux_control: STD_LOGIC_VECTOR(2 DOWNTO 0);

BEGIN

--condition evaluation
PROCESS(cond_id, start, msb_k, mult_done, count)
BEGIN
   CASE cond_id IS
      WHEN "000" => cond <= start;
      WHEN "001" => cond <= NOT(start);
      WHEN "010" => cond <= msb_k;
      WHEN "011" => cond <= NOT(mult_done);
      WHEN OTHERS => IF count = m-1 THEN cond <= '1'; ELSE cond <= '0'; END IF;
   END CASE;
END PROCESS;

--next state selection
mux_control <= u_type&cond;
WITH mux_control SELECT next_num <= 
   plus_1 WHEN "000",
   jump_num WHEN "010",
   plus_1 WHEN "100",
   jump_num WHEN OTHERS;

--half adder
plus_1 <= num + '1';

--state register
PROCESS(clk, reset)
BEGIN
   IF reset = '1' THEN num <= "000000";
   ELSIF clk'EVENT AND clk = '1' THEN
      num <= next_num;
   END IF;
END PROCESS; 

END circuit;
