----------------------------------------------------------------
-- seven_to_three.vhd
-- 
----------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.all;
ENTITY seven_to_three_seq IS
  GENERIC(n: NATURAL:= 8);
PORT (
  x1, x2, x3, x4, x5, x6, x7: IN STD_LOGIC_VECTOR(n-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  y1, y2, y3: OUT STD_LOGIC_VECTOR(n-1 DOWNTO 0)
);
END seven_to_three_seq;

ARCHITECTURE circuit OF seven_to_three_seq IS
  COMPONENT csa IS
    GENERIC(n: NATURAL);
  PORT (
    x1, x2, x3: IN STD_LOGIC_VECTOR(n-1 DOWNTO 0);
    y1, y2: OUT STD_LOGIC_VECTOR(n-1 DOWNTO 0)
  );
  END COMPONENT;
  SIGNAL r1, r2, s1, s2, xx1, xx2, xx3, yy1, yy2, xxx1, xxx2, xxx3, yyy1, yyy2: STD_LOGIC_VECTOR(n-1 DOWNTO 0);
  SIGNAL sel: STD_LOGIC;
  SIGNAL en01, en0, en2: STD_LOGIC;
  SIGNAL current_state: NATURAL RANGE 0 TO 2;
BEGIN

  WITH sel SELECT xx1 <= x4 WHEN '0', r1 WHEN OTHERS;   
  WITH sel SELECT xx2 <= x5 WHEN '0', r2 WHEN OTHERS;
  WITH sel SELECT xx3 <= x6 WHEN '0', x7 WHEN OTHERS;

  first_csa: csa GENERIC MAP(n => n)
  PORT MAP(x1 => xx1, x2 => xx2, x3 => xx3, y1 => yy1, y2 => yy2);

  first_register: PROCESS(clk)
  BEGIN
    IF clk'EVENT AND clk = '1' THEN 
      IF en01 = '1' THEN r1 <= yy1; r2 <= yy2; END IF;
    END IF;
  END PROCESS;

  WITH sel SELECT xxx1 <= x1 WHEN '0', s1 WHEN OTHERS;   
  WITH sel SELECT xxx2 <= x2 WHEN '0', s2 WHEN OTHERS;
  WITH sel SELECT xxx3 <= x3 WHEN '0', r1 WHEN OTHERS;

  second_csa: csa GENERIC MAP(n => n)
  PORT MAP(x1 => xxx1, x2 => xxx2, x3 => xxx3, y1 => yyy1, y2 => yyy2);

  second_register: PROCESS(clk)
  BEGIN
    IF clk'EVENT and clk = '1' THEN 
      IF en0 = '1' THEN s1 <= yyy1; s2 <= yyy2; END IF;
    END IF;
  END PROCESS;

--  y1 <= yyy1; y2 <= yyy2; y3 <= r2;

  output_register: PROCESS(clk)
  BEGIN
    IF clk'EVENT and clk = '1' THEN 
      IF en2 = '1' THEN y1 <= yyy1; y2 <= yyy2; y3 <= r2; END IF;
    END IF;
  END PROCESS;


  control_unit: PROCESS(clk, reset)
  BEGIN
    IF reset = '1' THEN current_state <= 0;
    ELSIF clk'event AND clk = '1' THEN
      IF current_state = 0 THEN current_state <= 1;
      ELSIF current_state = 1 THEN current_state <= 2;
      ELSIF current_state = 2 THEN current_state <= 0;
      END IF;
    END IF;
  END PROCESS;

  output_function: PROCESS(current_state)
  BEGIN
      IF current_state = 0 THEN en01 <= '1'; en0 <= '1'; sel <= '0'; en2 <= '0'; 
      ELSIF current_state = 1 THEN en01 <= '1'; en0 <= '0';  sel <= '1'; en2 <= '0';
      ELSIF current_state = 2 THEN en01 <= '0'; en0 <= '0';  sel <= '1'; en2 <= '1';
      END IF;
  END PROCESS;


END circuit;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.all;
ENTITY test_counter_seq IS END test_counter_seq;

ARCHITECTURE test OF test_counter_seq IS

COMPONENT seven_to_three_seq IS
  GENERIC(n: NATURAL:= 8);
PORT (
  x1, x2, x3, x4, x5, x6, x7: IN STD_LOGIC_VECTOR(n-1 DOWNTO 0);
  clk, reset: IN STD_LOGIC;
  y1, y2, y3: OUT STD_LOGIC_VECTOR(n-1 DOWNTO 0)
);
END COMPONENT;

  SIGNAL x1, x2, x3, x4, x5, x6, x7, y1, y2, y3: STD_LOGIC_VECTOR(7 DOWNTO 0);
  SIGNAL clk: STD_LOGIC := '0';
  SIGNAL reset: STD_LOGIC;

BEGIN
  dut: seven_to_three_seq GENERIC MAP(n => 8)
  PORT MAP(x1 => x1, x2 => x2, x3 => x3, x4 => x4, x5 => x5, x6 => x6, x7 => x7, 
  y1 => y1, y2 => y2, y3 => y3, clk => clk, reset => reset);

  clk <= NOT(clk) AFTER 50 ns;

  PROCESS
  BEGIN
    reset <= '1';
    x1 <= "00000001";
    x2 <= "00000010";
    x3 <= "00000011";
    x4 <= "00000000";
    x5 <= "00000001";
    x6 <= "00000010";
    x7 <= "00000011";
    WAIT FOR 100 ns;
    reset <= '0';
    WAIT FOR 250 ns;
    x1 <= "00010001";
    x2 <= "00010010";
    x3 <= "00010011";
    x4 <= "00010000";
    x5 <= "00010001";
    x6 <= "00010010";
    x7 <= "00010011";
    WAIT FOR 300 ns;
    x1 <= "00100001";
    x2 <= "00100010";
    x3 <= "00100011";
    x4 <= "00100000";
    x5 <= "00100001";
    x6 <= "00100010";
    x7 <= "00100011";
    WAIT FOR 300 ns;
    x1 <= "00110001";
    x2 <= "00110010";
    x3 <= "00110011";
    x4 <= "00110000";
    x5 <= "00110001";
    x6 <= "00110010";
    x7 <= "00110011";
    WAIT;
  END PROCESS;

END test;