LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
USE IEEE.MATH_REAL.ALL; -- for UNIFORM, TRUNC
ENTITY signal_generator IS 
  PORT(
    clk, reset, done: IN STD_LOGIC;
    random: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    start: OUT STD_LOGIC
  );
END signal_generator;

ARCHITECTURE behavior OF signal_generator IS

  TYPE state IS RANGE 0 TO 6;
  SIGNAL current_state: state;

  SIGNAL count: NATURAL;

BEGIN


next_state: PROCESS (clk, reset)
BEGIN
   IF reset = '1' THEN current_state <= 0;
   ELSIF clk'event AND clk = '1' THEN
      CASE current_state IS
         WHEN 0 => current_state <= 1;
         WHEN 1 => current_state <= 2;
         WHEN 2 => IF done = '0' THEN current_state <= 3; END IF;
         WHEN 3 => IF done = '1' THEN current_state <= 4; END IF;
         WHEN 4 => current_state <= 5;
         WHEN 5 => IF count /= 0 THEN current_state <= 6; END IF;
         WHEN 6 => current_state <= 2;
      END CASE;
   END IF;
END PROCESS;

operations: PROCESS (current_state)
--   VARIABLE seed1: NATURAL := 844396720;
--   VARIABLE seed2: NATURAL := 821616997;  
   VARIABLE seed1: NATURAL := 244396720;
   VARIABLE seed2: NATURAL := 221616997;  
   VARIABLE rand: REAL;
   VARIABLE int_random: NATURAL;
   VARIABLE next_random : STD_LOGIC_VECTOR(k-1 DOWNTO 0);
BEGIN
   CASE current_state IS
      WHEN 0 => random <= zero; start <= '0'; next_random := one; count <= 31;

--compute a random number < 91:
--        UNIFORM(seed1, seed2, rand);
--        int_random := INTEGER(TRUNC(rand*real(int_m)));
--        next_random := CONV_STD_LOGIC_VECTOR(int_random,k);

      WHEN 1 => start <= '1';  
      WHEN 2 => start <= '0'; 
      WHEN 3 => start <= '0'; 
      WHEN 4 => random <= next_random; start <= '0'; count <= count - 1;

        UNIFORM(seed1, seed2, rand);
        int_random := INTEGER(TRUNC(rand*real(91)));
        next_random := CONV_STD_LOGIC_VECTOR(int_random,k);

      WHEN 5 => start <= '1';     
      WHEN 6 => start <= '1';     
   END CASE;
END PROCESS;

END behavior;


LIBRARY IEEE; USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.mod_m.ALL;
ENTITY test_signal_generator IS END test_signal_generator;

ARCHITECTURE test OF test_signal_generator IS
COMPONENT signal_generator IS 
  PORT(
    clk, reset, done: IN STD_LOGIC;
    random: OUT STD_LOGIC_VECTOR(k-1 DOWNTO 0);
    start: OUT STD_LOGIC
  );
END COMPONENT;

SIGNAL random: STD_LOGIC_VECTOR(k-1 DOWNTO 0);
SIGNAL reset, start: STD_LOGIC;
SIGNAL clk, done: STD_LOGIC := '0';

BEGIN

dut: signal_generator
PORT MAP(random => random, clk => clk, reset => reset, start => start, done => done);

clk <= NOT(clk) AFTER 50 NS;

done <= NOT(done) AFTER 200 NS;

reset <= '1', '0' AFTER 100 NS;

END test;

