LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.coprocessor_parameters.ALL;
ENTITY system IS
PORT (
    clk: IN STD_LOGIC
);
END system;

ARCHITECTURE structure OF system IS

COMPONENT cpu IS
PORT (
    data_in: IN SIGNED(p-1 DOWNTO 0);
    clk, Grant: IN STD_LOGIC;
    data_out: OUT SIGNED(p-1 DOWNTO 0);
    address_out: OUT STD_LOGIC_VECTOR(s-1 DOWNTO 0); 
    ADS_out, write_out, Req: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT arbiter IS
PORT (
    clk, Req1, Req2: IN STD_LOGIC;
    Grant1, Grant2: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT coprocessor IS
PORT (
    data_in: IN SIGNED(p-1 DOWNTO 0);
    address_in: IN STD_LOGIC_VECTOR(s-1 DOWNTO 0); 
    clk, ADS_in, write_in, Grant: IN STD_LOGIC;
    data_out: OUT SIGNED(p-1 DOWNTO 0);
    address_out: OUT STD_LOGIC_VECTOR(s-1 DOWNTO 0); 
    ADS_out, write_out, Req: OUT STD_LOGIC
);
END COMPONENT;

COMPONENT data_memory IS
PORT (
    data_in: IN SIGNED(p-1 DOWNTO 0);
    address_in: IN STD_LOGIC_VECTOR(s-1 DOWNTO 0); 
    clk, ADS_in, write_in: IN STD_LOGIC;
    data_out: OUT SIGNED(p-1 DOWNTO 0)
);
END COMPONENT;

SIGNAL Req1, Grant1, Req2, Grant2: STD_LOGIC;
SIGNAL bus_data: SIGNED(p-1 DOWNTO 0); 
SIGNAL bus_address: STD_LOGIC_VECTOR(s-1 DOWNTO 0);
SIGNAL bus_write, bus_ADS: STD_LOGIC;

BEGIN

first_component: cpu PORT MAP(
    data_in => bus_data, clk => clk, Grant => Grant1, data_out => bus_data,
    address_out => bus_address, ADS_out => bus_ADS, write_out => bus_write, Req => Req1);

second_component: arbiter PORT MAP(
     clk => clk, Req1 => Req1, Req2 => Req2, Grant1 => Grant1, Grant2 => Grant2);

third_component: coprocessor PORT MAP(
    data_in => bus_data, address_in => bus_address, clk => clk, ADS_in => bus_ADS, 
    write_in => bus_write, Grant => Grant2, data_out => bus_data,
    address_out => bus_address, ADS_out => bus_ADS, write_out => bus_write, Req => Req2);

fourth_component: data_memory PORT MAP(
    data_in => bus_data, address_in => bus_address, clk => clk, 
    ADS_in => bus_ADS, write_in => bus_write, data_out => bus_data);

END structure;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.coprocessor_parameters.ALL;
ENTITY test_system IS END test_system;

ARCHITECTURE test OF test_system IS

COMPONENT system IS
PORT (
    clk: IN STD_LOGIC
);
END COMPONENT;

SIGNAL clk: STD_LOGIC := '1';

BEGIN

dut: system PORT MAP(
    clk => clk);

clk <= NOT(clk) AFTER 50 NS;

END test;
