library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
package main_parameters is
  constant m: natural := 32; -- m bits per word
  constant n: natural := 8; -- n words
  constant p: natural := 3; -- p address bits
  constant zero: std_logic_vector(m-1 downto 0) := conv_std_logic_vector(0, m);
end main_parameters;

library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity three_port_register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write_reg: in std_logic;
  address_A, address_B, address_W: in std_logic_vector(p-1 downto 0);
  data_out_A, data_out_B: out std_logic_vector(m-1 downto 0)  
);
end three_port_register_file;

architecture structure of three_port_register_file is

type memory is array (0 to n-1) of std_logic_vector(m-1 downto 0);
signal X: memory;
signal EN: std_logic_vector(0 to n-1);

begin

  decoder: process(address_W, write_reg)
  begin
    for i in 0 to n-1 loop
      if i = conv_integer(address_W) then EN(i) <= write_reg;
      else EN(i) <= '0';
      end if;
    end loop;
  end process;
  
   bank_registers: process(clk)
   begin
     if clk'event and clk = '1' then
       for i in 0 to n-1 loop
         if EN(i) = '1' then X(i) <= data_in; end if;
       end loop;
     end if;
    end process;

  multiplexer_A: process(address_A, X)
  begin
     for i in 0 to n-1 loop
        if i = conv_integer(address_A) then data_out_A <= X(i); end if;
     end loop;
   end process;  

  multiplexer_B: process(address_B, X)
  begin
     for i in 0 to n-1 loop
        if i = conv_integer(address_B) then data_out_B <= X(i); end if;
     end loop;
   end process;  


end structure;	


library ieee; 
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.main_parameters.all;
entity test_three_port_register_file is end test_three_port_register_file;

architecture test of test_three_port_register_file is

component three_port_register_file is
port (
  data_in: in std_logic_vector(m-1 downto 0);
  clk, write_reg: in std_logic;
  address_A, address_B, address_W: in std_logic_vector(p-1 downto 0);
  data_out_A, data_out_B: out std_logic_vector(m-1 downto 0)  
);
end component;

signal data_in: std_logic_vector(m-1 downto 0);
signal clk: std_logic := '0';
signal write_reg: std_logic;
signal address_A, address_B, address_W: std_logic_vector(p-1 downto 0);
signal data_out_A, data_out_B: std_logic_vector(m-1 downto 0);

begin

dut: three_port_register_file
port map(data_in => data_in, clk => clk, write_reg => write_reg,  
address_A => address_A, address_B => address_B, address_W => address_W, 
data_out_A => data_out_A, data_out_B => data_out_B);
  
clk <= not(clk) after 10 ns;

process
begin
write_reg <= '1';
data_in <= x"00000000";
address_W <= "000";

for j in 1 to n-1 loop
  wait for 100 ns;
  address_W <= address_W + 1;
  data_in <= data_in + 1;
end loop;

wait for 100 ns;
write_reg <= '0';
wait for 100 ns;

address_A <= "000"; address_B <= "010"; 

for j in 0 to n-1 loop
  wait for 100 ns;
  address_A <= address_A + 1; address_B <= address_B + 1; 
end loop;

wait;

end process;

end test;

