LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY uprogram_memory IS
PORT (
    address: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    u_type: OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    jump_num: OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    --command
    start_mult, load, en_xA, en_xB, en_zA, en_zB, en_R, shift, sel_R, done: OUT STD_LOGIC;
    sel_p1, sel_p2, sel_a1, sel_a2, sel_sq, sel_xA, sel_xB, sel_zA, sel_zB: 
       OUT STD_LOGIC_VECTOR(1 DOWNTO 0));
END uprogram_memory;

ARCHITECTURE behavior OF uprogram_memory IS
BEGIN

--ROM contents: next address
PROCESS(address)
   VARIABLE integer_address: NATURAL RANGE 0 TO 43;
BEGIN
   integer_address := CONV_INTEGER(address);   
   CASE integer_address IS
      WHEN 0 => u_type <= "10"; cond_id <= "000";  jump_num <= "000000";
      WHEN 1 => u_type <= "10"; cond_id <= "001";  jump_num <= "000001";
      WHEN 2 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 3 => u_type <= "10"; cond_id <= "010";  jump_num <= "010111";
      WHEN 4 TO 6 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 7 => u_type <= "10"; cond_id <= "011";  jump_num <= "000111";
      WHEN 8 TO 9 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 10 => u_type <= "10"; cond_id <= "011";  jump_num <= "001010";
      WHEN 11 TO 13 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 14 => u_type <= "10"; cond_id <= "011";  jump_num <= "001110";
      WHEN 15 TO 16 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 17 => u_type <= "10"; cond_id <= "011";  jump_num <= "010001";
      WHEN 18 TO 19 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 20 => u_type <= "10"; cond_id <= "011";  jump_num <= "010100";
      WHEN 21 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 22 => u_type <= "01"; cond_id <= "000";  jump_num <= "101010";
      WHEN 23 TO 25 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 26 => u_type <= "10"; cond_id <= "011";  jump_num <= "011010";
      WHEN 27 TO 28 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 29 => u_type <= "10"; cond_id <= "011";  jump_num <= "011101";
      WHEN 30 TO 32 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 33 => u_type <= "10"; cond_id <= "011";  jump_num <= "100001";
      WHEN 34 TO 35 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 36 => u_type <= "10"; cond_id <= "011";  jump_num <= "100100";
      WHEN 37 TO 38 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 39 => u_type <= "10"; cond_id <= "011";  jump_num <= "100111";
      WHEN 40 TO 41 => u_type <= "00"; cond_id <= "000";  jump_num <= "000000";
      WHEN 42 => u_type <= "10"; cond_id <= "100";  jump_num <= "000000";
      WHEN 43 => u_type <= "01"; cond_id <= "000";  jump_num <= "000011";
   END CASE;
END PROCESS;

--ROM contents: command
PROCESS(address)
   VARIABLE integer_address: NATURAL RANGE 0 TO 43;
BEGIN
  integer_address := CONV_INTEGER(address);   
  CASE integer_address IS
  WHEN 0 to 1 => 
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '1';
  WHEN 2 =>
    start_mult <= '0'; load <= '1'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 3 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 4 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "01"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 5 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; done <= '0';
  WHEN 6 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; done <= '0';
  WHEN 7 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 8 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 9 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 10 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 11 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 12 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "10"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 13 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 14 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 15 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 16 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "01"; sel_p2 <= "10"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "10"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; done <= '0';
  WHEN 17 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 18 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 19 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 20 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 21 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 22 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "10"; 
    sel_xB <= "01"; sel_zA <= "10"; sel_zB <= "10"; sel_R <= '0'; done <= '0';
  WHEN 23 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "00"; sel_a1 <= "01"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 24 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 25 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '1'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "01"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 26 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 27 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 28 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 29 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 30 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 31 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "01"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "10"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 32 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "01"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "01"; sel_R <= '0'; done <= '0';
  WHEN 33 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 34 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 35 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '1'; sel_p1 <= "00"; sel_p2 <= "10"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "11"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '1'; done <= '0';
  WHEN 36 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 37 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 38 =>
    start_mult <= '1'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "10"; sel_p2 <= "01"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 39 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 40 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '1'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 41 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '1'; en_xB <= '1'; en_zA <= '1'; en_zB <= '1'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "01"; 
    sel_xB <= "10"; sel_zA <= "11"; sel_zB <= "11"; sel_R <= '0'; done <= '0';
  WHEN 42 =>
    start_mult <= '0'; load <= '0'; shift <= '0'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  WHEN 43 =>
    start_mult <= '0'; load <= '0'; shift <= '1'; en_xA <= '0'; en_xB <= '0'; en_zA <= '0'; en_zB <= '0'; 
    en_R <= '0'; sel_p1 <= "00"; sel_p2 <= "00"; sel_a1 <= "00"; sel_a2 <= "00"; sel_sq <= "00"; sel_xA <= "00"; 
    sel_xB <= "00"; sel_zA <= "00"; sel_zB <= "00"; sel_R <= '0'; done <= '0';
  END CASE;
END PROCESS;
END behavior;
