LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE WORK.scalar_product_parameters.ALL;
ENTITY uprogram_memory2 IS
PORT (
    address: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
    u_type: OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    cond_id: OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
    num_mnemo: OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
    done: OUT STD_LOGIC);
END uprogram_memory2;

ARCHITECTURE behavior OF uprogram_memory2 IS

CONSTANT nop: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00000";
CONSTANT sw_reset: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00001";
CONSTANT multAB1: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00010";
CONSTANT squareB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00011";
CONSTANT updateR: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00100";
CONSTANT multBA1: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00101";
CONSTANT updateB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00110";
CONSTANT multAA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "00111";
CONSTANT squareA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01000";
CONSTANT updateA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01001";
CONSTANT multRB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01010";
CONSTANT multPA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01011";
CONSTANT updateAB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01100";
CONSTANT multBA2: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01101";
CONSTANT multAB2: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01110";
CONSTANT multBB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "01111";
CONSTANT multRA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10000";
CONSTANT multPB: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10001";
CONSTANT updateBA: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10010";
CONSTANT inc: STD_LOGIC_VECTOR(4 DOWNTO 0) := "10011";

BEGIN

--ROM contents: next address or mnemonic
PROCESS(address)
   VARIABLE integer_address: NATURAL RANGE 0 TO 45;
BEGIN
   integer_address := CONV_INTEGER(address);   
   CASE integer_address IS
      WHEN 0 => u_type <= "10"; cond_id <= "000";  num_mnemo <= "000000"; done <= '1';
      WHEN 1 => u_type <= "10"; cond_id <= "001";  num_mnemo <= "000001"; done <= '1';
      WHEN 2 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&sw_reset; done <= '0';
      WHEN 3 => u_type <= "10"; cond_id <= "010";  num_mnemo <= "011000"; done <= '0';
      WHEN 4 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multAB1; done <= '0';
      WHEN 5 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&squareB; done <= '0';
      WHEN 6 => u_type <= "00"; cond_id <= "011";  num_mnemo <= '0'&squareB; done <= '0';
      WHEN 7 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "000111"; done <= '0';
      WHEN 8 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateR; done <= '0';
      WHEN 9 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multBA1; done <= '0';
      WHEN 10 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "001010"; done <= '0';
      WHEN 11 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateB; done <= '0';
      WHEN 12 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multAA; done <= '0';
      WHEN 13 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&squareA; done <= '0';
      WHEN 14 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "001110"; done <= '0';
      WHEN 15 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateA; done <= '0';
      WHEN 16 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multRB; done <= '0';
      WHEN 17 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "010001"; done <= '0';
      WHEN 18 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateB; done <= '0';
      WHEN 19 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multPA; done <= '0';
      WHEN 20 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "010100"; done <= '0';
      WHEN 21 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateA; done <= '0';
      WHEN 22 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateAB; done <= '0';
      WHEN 23 => u_type <= "01"; cond_id <= "000";  num_mnemo <= "101011"; done <= '0';
      WHEN 24 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multBA2; done <= '0';
      WHEN 25 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&squareA; done <= '0';
      WHEN 26 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&squareA; done <= '0';
      WHEN 27 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "011011"; done <= '0';
      WHEN 28 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateR; done <= '0';
      WHEN 29 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multAB2; done <= '0';
      WHEN 30 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "011110"; done <= '0';
      WHEN 31 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateA; done <= '0';
      WHEN 32 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multBB; done <= '0';
      WHEN 33 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&squareB; done <= '0';
      WHEN 34 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "100010"; done <= '0';
      WHEN 35 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateB; done <= '0';
      WHEN 36 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multRA; done <= '0';
      WHEN 37 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "100101"; done <= '0';
      WHEN 38 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateA; done <= '0';
      WHEN 39 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&multPB; done <= '0';
      WHEN 40 => u_type <= "10"; cond_id <= "011";  num_mnemo <= "101000"; done <= '0';
      WHEN 41 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateB; done <= '0';
      WHEN 42 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&updateBA; done <= '0';
      WHEN 43 => u_type <= "10"; cond_id <= "100";  num_mnemo <= "000000"; done <= '0';
      WHEN 44 => u_type <= "00"; cond_id <= "000";  num_mnemo <= '0'&inc; done <= '0';
      WHEN 45 => u_type <= "01"; cond_id <= "000";  num_mnemo <= "000011"; done <= '0';
   END CASE;
END PROCESS;

END behavior;
